# Zope imports
from zope.interface import implements
from zope.formlib import form

# plone imports
from Products.statusmessages.interfaces import IStatusMessage
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

# Quills imports
from quills.core.browser.weblogconfig import WeblogConfigAnnotations
from quills.core.browser.weblogconfig import WeblogConfigEditForm
from quills.app.interfaces import IStateAwareWeblogConfiguration
from quills.app.interfaces import IWeblogEnhancedConfiguration
from quills.app import QuillsAppMessageFactory as _

class StateAwareWeblogConfig(WeblogConfigAnnotations):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogEnhancedConfiguration, StateAwareWeblogConfig)
    True
    """

    implements(IWeblogEnhancedConfiguration)

    default_type = "WeblogEntry"

    def _get_published_states(self):
        return self._config.get('published_states', ['published',])
    def _set_published_states(self, value):
        self._config['published_states'] = value
    published_states = property(_get_published_states, _set_published_states)

    def _get_draft_states(self):
        return self._config.get('draft_states', ['private',])
    def _set_draft_states(self, value):
        self._config['draft_states'] = value
    draft_states = property(_get_draft_states, _set_draft_states)


class StateAwareWeblogConfigEditForm(WeblogConfigEditForm):
    """Edit form for weblog view configuration.
    """
    label = _(u'Configure Blog')
    description = _(u"Weblog View Configuration")
    form_name = _(u"Configure rule")
    template = ViewPageTemplateFile('weblogconfig.pt')

    # We use IStateAwareWeblogConfiguration instead of
    # IWeblogEnhancedConfiguration because we don't want to generate form
    # input for the default_type field.
    form_fields = form.Fields(IStateAwareWeblogConfiguration)


    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        wvconfig = IStateAwareWeblogConfiguration(self.context)
        self.widgets = form.setUpEditWidgets(
            self.form_fields, self.prefix, wvconfig, self.request,
            adapters=self.adapters, ignore_request=ignore_request
            )

    @form.action("submit")
    def submit(self, action, data):
        """ """
        wvconfig = IStateAwareWeblogConfiguration(self.context)
        form.applyChanges(wvconfig, self.form_fields, data)
        msg = _(u'Configuration saved.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')




# Zope imports
from zope.interface import implements
from zope.publisher.interfaces.browser import IBrowserPublisher


class DefaultView(object):
    """Provides the name of a default view. Abstract, subclasses
    must provide the name.
    
    Used by the Zope publisher during traversal.
    """
    implements(IBrowserPublisher)
    
    def __init__(self, name, context, request):
	self.name = name
	self.context = context
	self.request = request

    def publishTraverse(self, request, name):
        """See zope.publisher.interfaces.IPublishTraverse
        
        This is not really needed as browserDefault returns our
        context and not ourselves. 
        """
        return self.context.publishTraverse(request, name)

    def browserDefault(self, request):
        """See  zope.publisher.interfaces.IBrowserPublisher"""
        return (self.context, (self.name,))

class DefaultArchiveView(DefaultView):
    """Provide the name of the archive default view.
    """
    def __init__(self, context, request):
        super(DefaultArchiveView, self).__init__("@@archive_view",
                                                context, request)

class DefaultTopicsView(DefaultView):
    """Provide the name of the topic listing's default view.
    """
    def __init__(self, context, request):
        super(DefaultTopicsView, self).__init__("@@topic_listing",
                                                context, request)

class DefaultAuthorsView(DefaultView):
    """Provide the name of the authors listing's default view.
    """
    def __init__(self, context, request):
        super(DefaultAuthorsView, self).__init__("@@author_listing",
                                                context, request)



# Zope imports
from zope.interface import implements

# CMF imports
from Products.CMFCore.utils import getToolByName

try:
    ## for Plone 4
    from Products.CMFCore.interfaces import IActionProvider
except ImportError:
    ## for Plone 3
    from Products.CMFCore.interfaces.portal_actions import ActionProvider as IActionProvider

class AcquiringActionProvider:

    implements(IActionProvider)

    def listActions(self, info=None, object=None):
        """ List all the actions defined by a provider.

        If 'object' is specified, object specific actions are included.

        The 'info' argument is deprecated and may be removed in a future
        version. If 'object' isn't specified, the method uses for backwards
        compatibility 'info.content' as object.

        Returns -- Tuple of ActionInformation objects (or Action mappings)
        """
        raise NotImplementedError

    def getActionObject(self, action):
        """Return the actions object or None if action doesn't exist.
        
        'action' is an action 'path' (e.g. 'object/view').
        
        Raises an ValueError exception if the action is of the wrong format.
        
        Permission -- Private
        
        Returns -- The actions object reference.
        """
        raise NotImplementedError

    def listActionInfos(self, action_chain=None, object=None, check_visibility=1,
                        check_permissions=1, check_condition=1, max=-1):
        """ List ActionInfo objects.

        'action_chain' is a sequence of action 'paths' (e.g. 'object/view').
        If specified, only these actions will be returned in the given order.

        If 'object' is specified, object specific Actions are included.

        If 'max' is specified, only the first max Actions are returned.

        Permission -- Always available (not publishable)

        Returns -- Tuple of ActionInfo objects
        """
        atool = getToolByName(self, 'portal_actions')
        factions = atool.listFilteredActionsFor(self.aq_parent)
        actions = []
        for key in factions.keys():
            # We filter out most of the categories of actions as these will
            # get added anyway, and we don't want duplicates.
            if key in ['object_tabs', 'object']:
                actions.extend(factions[key])
        # This is a bit hacky.  We step through the actions and remove the
        # 'Contents' one as Plone will automatically add that later.
        actions2 = []
        for action in actions:
            if action['title'] != "Contents":
                actions2.append(action)
        return actions2


    def getActionInfo(self, action_chain, object=None, check_visibility=0,
                      check_condition=0):
        """ Get an ActionInfo object specified by a chain of actions.

        Permission -- Always available

        Returns -- ActionInfo object
        """
        raise NotImplementedError



from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.app.portlets.browser.formhelper import NullAddForm
from plone.memoize.compress import xhtml_compress
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

# Quills imports
from quills.core.interfaces import IWeblogEnhanced
from quills.core.interfaces import IWeblog
from quills.app.utilities import recurseToInterface
from quills.app import QuillsAppMessageFactory as _

# Local imports
from base import BasePortletRenderer


PORTLET_TITLE = _(u"Quills")
PORTLET_DESC = _(u"This portlet provides links to the various feeds of this instance.")

class IQuillsLinksPortlet(IPortletDataProvider):
    """A weblog administration portlet.
    """


class Assignment(base.Assignment):
    implements(IQuillsLinksPortlet)

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer):

    _template = ViewPageTemplateFile('quillslinks.pt')

    @property
    def title(self):
        return PORTLET_TITLE

    @property
    def portal_url(self):
        return getToolByName(self.context, 'portal_url')


class AddForm(NullAddForm):
    form_fields = form.Fields(IQuillsLinksPortlet)
    label = _(u'add-portlet', default=u"Add ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self):
        return Assignment()


class EditForm(base.EditForm):
    form_fields = form.Fields(IQuillsLinksPortlet)
    label = _(u'edit-portlet', default=u"Edit ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = _(PORTLET_DESC)



from zope import schema
from zope.formlib import form
from zope.interface import implements
from zope.component import getMultiAdapter

from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from quills.app.utilities import getArchiveURLFor
from quills.app.browser.baseview import BaseView
from quills.app import QuillsAppMessageFactory as _

from base import BasePortletRenderer

PORTLET_TITLE = _(u"Recent Comments")
PORTLET_DESC = _(u"This portlet lists recent weblog comments.")


class IRecentWeblogCommentsPortlet(IPortletDataProvider):
    max_comments = schema.Int(
        title=_(u'Maximum comments'),
        description=_(u"What's the maximum number of comments to list?"),
        required=True,
        default=5)


class Assignment(base.Assignment):
    implements(IRecentWeblogCommentsPortlet)

    def __init__(self, max_comments=5):
        self.max_comments = max_comments

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer, BaseView):
    _template = ViewPageTemplateFile('recentcomments.pt')

    @property
    def available(self):
        return len(self.getComments) > 0

    @property
    def title(self):
        return PORTLET_TITLE

    @property
    def getComments(self):
        weblog_content = self.getWeblogContentObject()
        if weblog_content is None:
            return []
        view = getMultiAdapter((weblog_content, self.request), name='manage_comments')
        return view.getComments()[:self.data.max_comments]

    def talkbackURL(self, item):
        # XXX This is (sadly) CMF-DiscussionItem-specific :(.
        comment = item.getObject()
        if not hasattr(comment, 'parentsInThread'):
            return comment.absolute_url()
        parent_comments = comment.parentsInThread()
        commented_object = parent_comments[0]
        weblog_content = self.getWeblogContentObject()
        base_url = getArchiveURLFor(commented_object, weblog_content)
        return '%s#%s' % (base_url, item.id)


class AddForm(base.AddForm):
    form_fields = form.Fields(IRecentWeblogCommentsPortlet)
    label = _(u'add-portlet', 
            default=u"Add ${portlet-name} Portlet", 
            mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self, data):
        return Assignment(max_comments=5)


class EditForm(base.EditForm):
    form_fields = form.Fields(IRecentWeblogCommentsPortlet)
    label = _(u'edit-portlet', 
            default=u"Edit ${portlet-name} Portlet", 
            mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC





# Zope 3 imports
from zope.interface import implements
from zope.component import getMultiAdapter

# Zope 2 imports
from DateTime.DateTime import DateTime

# CMF imports
from Products.CMFCore.utils import getToolByName

# fatsyndication imports
from Products.fatsyndication.adapters import BaseFeed
from Products.fatsyndication.adapters import BaseFeedSource
from Products.fatsyndication.adapters import BaseFeedEntry
from Products.basesyndication.interfaces import IFeedSource
from Products.basesyndication.interfaces import IFeedEntry
from quills.core.interfaces import IWeblog, IWeblogEntry

# for workflow awareness
from quills.app.interfaces import IWeblogEnhancedConfiguration

class WeblogFeed(BaseFeed):
    """Adapter from Quills Weblog instances to IFeed
    """

    def getModifiedDate(self):
        """See IFeed.
        """
        # Quills has the luxury of being able to compute the effective modification date
        # in a much more efficient manner, i.e. by querying the catalog:
        path = '/'.join(self.context.getPhysicalPath())
        weblog_config = IWeblogEnhancedConfiguration(self.context)
        results = self.context.portal_catalog(
            meta_type=['WeblogEntry', 'Discussion Item',],
            path={'query':path, 'level': 0},
            sort_on = 'modified',
            sort_order = 'reverse',
            review_state = weblog_config.published_states)

        # just like fatsyndication, we return 'now', if there
        # aren't any entries:
        try:
            return results[0].getObject().modified()
        except IndexError:
            return DateTime()



class WeblogFeedSource(BaseFeedSource):
    """Adapter from Quills Weblog instances to IFeedSource
    """

    implements(IFeedSource)

    def __init__(self, context):
        self.context = IWeblog(context)

    def getFeedEntries(self, max_only=True):
        """See IFeedSoure
        """
        if max_only:
            num_of_entries = self.getMaxEntries()
        else:
            num_of_entries = 0 # signals to fetch _all_ items
        brains = self.context.getEntries(num_of_entries)
        return [IFeedEntry(brain.getObject()) for brain in brains]

    # Quills always returns sorted entries, so we can override fatsyndications
    # expensive (but generic) implementation:
    getSortedFeedEntries = getFeedEntries


class TopicFeed(BaseFeed):
    pass


class TopicFeedSource(BaseFeedSource):
    """Adapter from Quills Topic instances to IFeedSource
    """

    implements(IFeedSource)

    def __init__(self, context):
        self.context = context

    def getFeedEntries(self):
        """See IFeedSoure
        """
        brains = self.context.getEntries()
        return [IFeedEntry(brain.getObject()) for brain in brains]


class WeblogEntryFeed(BaseFeed):
    pass


class WeblogEntryFeedSource(BaseFeedSource):
    """An adapter from IWeblogEntry to IFeedSource that feeds out the
    entry's comments.
    """

    implements(IFeedSource)

    def __init__(self, context):
        self.context = IWeblogEntry(context)

    def getFeedEntries(self):
        """See IFeedSource
        """
        d_tool = getToolByName(self.context, 'portal_discussion')
        if d_tool.isDiscussionAllowedFor(self.context):
            discussion = d_tool.getDiscussionFor(self.context)
            replies = [IFeedEntry(reply) for reply in discussion.getReplies()]
        else:
            replies = []
        return [IFeedEntry(self.context)] + replies


class WeblogEntryFeedEntry(BaseFeedEntry):
    """
    """

    implements(IFeedEntry)

    def getXhtml(self):
        html_types = ('text/html', 'text/x-rst',
                      'text/restructured', 'text/structured')
        if self.context.text.mimetype in html_types:
            return self.context.getText()
        else:
            return None

    def getWebURL(self):
        """See IFeedEntry.
        """
        we_view = getMultiAdapter((self.context, self.context.REQUEST),
                                  name='weblogentry_view')
        return we_view.getArchiveURLFor(self.context)

    def getBody(self):
        """See IFeedEntry.
        """
        return self.context.getText()

    def getModifiedDate(self):
        """See IFeedEntry.
        """

        # we override fatsyndications generic implementation and compare the 
        # entry's modification date with those of any comments:
        modified = self.context.modified()

        path = '/'.join(self.context.getPhysicalPath())
        weblog_config = IWeblogEnhancedConfiguration(self.context.aq_inner.aq_parent)
        results = self.context.portal_catalog(
            meta_type=['Discussion Item',],
            path={'query':path, 'level': 0},
            sort_on = 'modified',
            sort_order = 'reverse',
            review_state = weblog_config.published_states)

        try:
            comment_mod = results[0].getObject().modified()
            if comment_mod > modified:
                modified = comment_mod
        except IndexError:
             pass
        return modified

def selectFeedAdapter(context):
    """Create an apropriate IWeblogEntry to IFeedEntry adapter.based on
    whether the context object is a WeblogEntry or an arbitrary portal
    type used by QuillsEnabled.
    """
    if hasattr(context, 'context'):
        try:
            return IFeedEntry(context.context)
        except ComponentLookupError:
            pass
    return WeblogEntryFeedEntry(context)



# Zope imports
from zope.interface import implements
from Products.Five import BrowserView

# Plone imports
from Products.CMFCore.utils import getToolByName

# Quills imports
from quills.core.interfaces import IWeblogArchive, ITopic, IWeblog, IWeblogEntry
from quills.core.browser.interfaces import IBaseView
from quills.app.utilities import getArchivePathFor, getArchiveURLFor


class BaseView(BrowserView):
    """A class with helper methods for use in views/templates.
    """

    implements(IBaseView)

    def isWeblogContent(self, obj=None):
        """See IBaseView.
        """
        if obj is None:
            obj = self.context
        if IWeblog.providedBy(obj):
            return True
        elif IWeblogArchive.providedBy(obj):
            return True
        elif IWeblogEntry.providedBy(obj):
            return True
        elif ITopic.providedBy(obj):
            return True
        return False

    def getArchivePathFor(self, obj):
        """See IWeblogView.
        """
        return getArchivePathFor(obj)

    def getArchiveURLFor(self, obj):
        """See IWeblogView.
        """
        weblog_content = obj.getWeblogContentObject()
        return getArchiveURLFor(obj, weblog_content)

    def displayingOneEntry(self, context, weblogentry):
        """See IWeblogView.
        """
        entry_absolute_url = getattr(weblogentry, 'absolute_url', None)
        if entry_absolute_url is None:
            entry_absolute_url = weblogentry.context.absolute_url
        return context.absolute_url() == entry_absolute_url()

    def isDiscussionAllowedFor(self, obj):
        """
        """
        dtool = getToolByName(self.context, 'portal_discussion')
        return dtool.isDiscussionAllowedFor(obj.getWeblogEntryContentObject())

    def getCommentCountFor(self, obj):
        """
        """
        dtool = getToolByName(self.context, 'portal_discussion')
        obj = obj.getWeblogEntryContentObject()
        return dtool.getDiscussionFor(obj).replyCount(obj)



import unittest
from doctest import DocTestSuite

# Zope imports
import zope.component.testing

def setUp(test):
    pass

suites = (
    DocTestSuite('quills.app.topic',
                 setUp=zope.component.testing.setUp,
                 tearDown=zope.component.testing.tearDown),
    DocTestSuite('quills.app.archive',
                 setUp=zope.component.testing.setUp,
                 tearDown=zope.component.testing.tearDown),
    DocTestSuite('quills.app.weblogentrybrain',
                 setUp=zope.component.testing.setUp,
                 tearDown=zope.component.testing.tearDown),
    DocTestSuite('quills.app.browser.weblogview',
                 setUp=zope.component.testing.setUp,
                 tearDown=zope.component.testing.tearDown),
    )

def test_suite():
    return unittest.TestSuite(suites)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')


# zope imports
from zope.interface import implements
#from zope.interface import Interface
#from zope.component import adapts

# plone imports
from plone.portlets.interfaces import IPortletContext
from plone.app.portlets.portletcontext import ContentContext

# quills imports
from quills.core.interfaces import IBaseContent
from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogEnhanced


INTERFACE_CATEGORY = 'interface_category'


class WeblogAwarePortletContext(ContentContext):
    """
    """

    implements(IPortletContext)

    def globalPortletCategories(self, placeless=False):
        cats = super(WeblogAwarePortletContext, self).globalPortletCategories(placeless)
        #ifaces = self._getInterfaces()
        ifaces = [IWeblog,]
        for iface in ifaces:
            cats.append((INTERFACE_CATEGORY, iface.__identifier__))
        return cats



from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.app.portlets.browser.formhelper import NullAddForm
from plone.memoize.compress import xhtml_compress
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneLocalesMessageFactory as _PLMF

# Quills imports
from quills.core.interfaces import IWeblogEnhanced
from quills.core.interfaces import IWeblog
from quills.app.utilities import recurseToInterface
from quills.app.browser.baseview import BaseView
from quills.app import QuillsAppMessageFactory as _

# Local imports
from base import BasePortletRenderer


PORTLET_TITLE = _(u"Weblog Archive")
PORTLET_DESC = _(u"This portlet lists the archive of this weblog.")


class IWeblogArchivePortlet(IPortletDataProvider):
    """a portlet showing the archive of the current weblog"""


class Assignment(base.Assignment):

    implements(IWeblogArchivePortlet)

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer, BaseView):

    _template = ViewPageTemplateFile('archive.pt')
    
    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.request = request
        self.view = view
        self.manager = manager
        self.data = data
        self._translation_service = getToolByName(self.context, 'translation_service')

    def getMonthName(self, month):
        """Returns the current month name as a Message."""
        msgid   = self._translation_service.month_msgid(month)
        english = self._translation_service.month_english(month)
        return _PLMF(msgid, default=english)

    @property
    def title(self):
        return PORTLET_TITLE

    @property
    def getSubArchives(self):
       weblog = self.getWeblog()
       return weblog.getSubArchives()


class AddForm(NullAddForm):
    form_fields = form.Fields(IWeblogArchivePortlet)
    label = _(u'add-portlet', default=u"Add ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self):
        return Assignment()


class EditForm(base.EditForm):
    form_fields = form.Fields(IWeblogArchivePortlet)
    label = _(u'edit-portlet', default=u"Edit ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC



# Plone imports
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize

# Quills imports
from quills.core.interfaces import IWeblogLocator


class BasePortletRenderer:

    #@ram.cache(render_cachekey)
    def render(self):
        return xhtml_compress(self._template())

    @property
    def available(self):
        return self.getWeblog() != [] and True or False

    @memoize
    def getWeblog(self):
        locator = IWeblogLocator(self.context)
        weblog = locator.find()
        return weblog

    @memoize
    def getWeblogContentObject(self):
        weblog = self.getWeblog()
        if getattr(weblog, 'absolute_url', None):
            return weblog
        elif getattr(weblog, 'context', None):
            # `weblog' is presumably an adapter around the real content object.
            return weblog.context



# -*- coding: utf-8 -*-
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import getUtility
try:
    from zope.container.interfaces import INameChooser  # Plone >= 4.1
except ImportError:
    from zope.app.container.interfaces import INameChooser


class WeblogMixin:
    """A mixin class with some helper methods for implementing
    quills.core.interfaces.IWeblog.
    """

    def _getKeywordsForBlogEntries(self):
        """Return a sequence of all keywords that are associatd with
        IWeblogEntry instances contained in this IWeblog.
        """
        entries = self.getAllEntries()
        # Use dict rather than list to avoid duplicates
        keywords = {}
        for entry in entries:
            for kw in entry.Subject:
                keywords[kw] = None
        keys = keywords.keys()
        keys.sort()
        return keys

    def _filter(self, results, maximum=None, offset=0):
        if len(results) > offset:
            if maximum is None:
                return results[offset:]
            else:
                return results[offset:offset + maximum]
        return results

    def _genUniqueId(self, folder, id, title):
        # XXX Add test-case for id generation (e.g. via addFile)
        idGen = getUtility(IIDNormalizer)
        if id:
            id = idGen.normalize(id)
        elif title:
            id = idGen.normalize(title)
        if not folder.hasObject(id):
            return id
        else:
            # Fall back to auto-gen
            return INameChooser(folder).chooseName(name='', object=None)

    def _getPortalTypeForMimeType(self, mimetype):
        # XXX Implement me properly!
        if 'image' in mimetype:
            return 'Image'
        return 'File'



# Standard library imports
from types import ListType
from types import TupleType
from types import StringTypes
import re

# to workaround slice incompatibility in Zope 2
from sys import maxint

# Zope imports
from Acquisition import Explicit
from Acquisition import aq_parent

# Plone imports
from Products.CMFCore.interfaces import ISiteRoot

# Quills imports
from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogEnhanced
from quills.core.interfaces import IWeblogConfiguration
from quills.core.interfaces import IWeblogEntry


class EvilAATUSHack(Explicit):

    def __call__(self, name, value):
        """Deny access to attributes that are found on the class or instance,
        but allow access to anything else.  This locks down security on object
        attributes, but allows the lookup of portlets with the 'provider'
        TAL command.
        """
        context = aq_parent(self)
        idict = context.__dict__
        cdict = context.__class__.__dict__
        if idict.has_key(name) or cdict.has_key(name):
            return 0
        return 1


class QuillsMixin(object):
    """
    """

    def getWeblogContentObject(self):
        return recurseToInterface(self, (IWeblog, IWeblogEnhanced))

    def getWeblog(self):
        obj = self.getWeblogContentObject()
        if IWeblog.providedBy(obj):
            return obj
        return IWeblog(obj)


def getArchivePathFor(obj, weblog_content):
    """See IWeblogView.
    """
    # Handle published and getId either being an attribute on a catalog brain or
    # a method on a content object.
    id = getattr(obj, 'getId')
    if callable(id):
        id = id()
    published = getattr(obj, 'effective', None)
    if published is None:
        published = getattr(obj, 'getPublicationDate')
    if callable(published):
        published = published() 
    path = []
    weblog_config = IWeblogConfiguration(weblog_content)
    archive_format = weblog_config.archive_format
    if isinstance(archive_format, StringTypes):
        archive_format = archive_format.strip()
    if archive_format is not None and archive_format is not '':
        path.append(archive_format)
    path.append(published.strftime('%Y'))
    path.append(published.strftime('%m'))
    path.append(published.strftime('%d'))
    path.append(id)
    return path

def getArchiveURLFor(obj, weblog_content):
    """See IWeblogView.
    """
    archive_path = getArchivePathFor(obj, weblog_content)
    return '%s/%s' % (weblog_content.absolute_url(), '/'.join(archive_path))

def recurseToInterface(item, ifaces):
    """Recurse up the aq_chain until an object providing `iface' is found,
    and return that.
    """
    if not isinstance(ifaces, (ListType, TupleType)):
        ifaces = [ifaces]
    parent = item.aq_inner.aq_parent
    for iface in ifaces:
        if iface.providedBy(item):
            return item
    for iface in ifaces:
        if iface.providedBy(parent):
            return parent
    for iface in ifaces:
        if ISiteRoot.providedBy(parent):
            # Stop when we get to the portal root.
            return None
    return recurseToInterface(parent, ifaces)


talkback_url_extractor = re.compile("(.*)/talkback/\d+")

def talkbackURL(discussion_brain):
    """expects the brain of a discussion item and constructs a url for it.
    n.b. we're using a regex in order to allow for the string 'talkback' to appear
    in the url in other places, too.
    """
    url = discussion_brain.getURL()
    absolute_url = talkback_url_extractor.search(url).groups()[0]
    return "%s#%s" % (absolute_url, discussion_brain.id)

class BloggifiedCatalogResults(object):
    """Wrap the CatalogBrains of the given iterable collection into
       IWeblogEntry adapters. All the elemets of catalogBrain must be
       wrappable by the same adapter class for reasons of optimization!
    """

    # A cached instance of the IWeblogEntry adapter to be used for
    # wrapping the catalog brains.
    _adapter = None

    def __init__(self, results):
        self.context = results
        # Cache an instance of the IWeblogAdapter which is to be used
        # for wrapping the brains. On class level for now. This expects
        # that all instances passed to this class actually can be handled
        # by the same wrapper!
        if BloggifiedCatalogResults._adapter is None and len(results) > 0:
            BloggifiedCatalogResults._adapter = IWeblogEntry(results[0])
            BloggifiedCatalogResults._adapter.context = None

    def __iter__(self):
        def iter(seq):
            for brain in self.context:
                yield BloggifiedCatalogResults.wrap(brain)
            raise StopIteration
        return iter(self.context)


    def __getitem__(self, k):
        """Return the item a index k or the slice indicated by k,
           the former as a IWeblogEntry the latter as an instance
           of this class.
        """
        if isinstance(k, slice):
            # Work around pre Python 2 collection code in Zope 2,
            # namely Products.ZCatalog.Lazy. Those will break on
            # slices. Why cannot things be simple for once?
            theSlice = None
            if k.step is None and hasattr(self.context, '__getslice__'):
                start = k.start
                stop = k.stop
                if start is None:
                    start = 0
                if stop is None:
                    stop = maxint
                theSlice = self.context[start:stop]
            else:
                theSlice = self.context[k]
                
            return BloggifiedCatalogResults(theSlice)
        else:
            return BloggifiedCatalogResults.wrap(self.context[k])

    def __len__(self):
        return len(self.context)

    def __contains__(self, item):
        return item in self.context
            
    def wrap(brain):
        """Wrap a catalog brain within an IWeblogEntry adapter.
        
        The _adater attribute of this instance is expected to hold
        an appropriate adapter instance for copying!
        """
        # adapter must be pickable for copy.copy to work
        # from copy import copy
        # adapter = copy(BloggifiedCatalogResults._adapter)
        # adapter.context = brain
        adapter = BloggifiedCatalogResults._adapter.__class__(brain)
        # Restricted code might (alas, and will) access this adapter,
        # that's why we will need to acquire the users roles.
        # XXX: Remove calls to IWeblogEntry from page templates
        #      that do are not explicitly allowed to access it!
        return adapter.__of__(brain)
    
    wrap = staticmethod(wrap)
        



from zope.interface import implements
from zope.component import getUtility
from Products.CMFCore.interfaces import ISiteRoot
from quills.core.interfaces import IWeblogEnhanced
from quills.core.interfaces import IWeblog
from quills.app.utilities import recurseToInterface
from quills.app.traversal import IInsideWeblog
from quills.core.interfaces import IWeblogLocator


class RecursingUpLocator(object):
    """see Interface, recurses up"""
    implements(IWeblogLocator)
    
    def __init__(self, context):
        self.context = context
        
    def find(self):
        """see Interface"""
        # The request tells us if were inside a weblog; if we are
        # not, we leave immediately. --- jhackel
        request = getattr(self.context, 'request', None)
        if not ( request is None or IInsideWeblog.providedBy(request) ):
            return []

        weblog_content = recurseToInterface(self.context.aq_inner,
                                            (IWeblog, IWeblogEnhanced))
        if weblog_content is None:
            return []
        return IWeblog(weblog_content)

class SelfLocator(object):
    """see Interface, takes always the context itself."""
    implements(IWeblogLocator)
    
    def __init__(self, context):
        self.context = context
        
    def find(self):
        """see Interface"""
        assert(IWeblog.providedBy(self.context)) 
        return self.context



class LatestFromCatalogLocator(object):
    """see Interface, uses the catalog."""
    implements(IWeblogLocator)
    
    def __init__(self, context):
        self.context = context
        
    def find(self):
        """see Interface"""
        site = getUtility(ISiteRoot, context=self.context)
        catalog = site.portal_catalog
        query = {
            'object_provides': 'quills.core.interfaces.weblog.IWeblog',
            'sort_on': 'effective',
            'sort_order': 'reverse',
        }
        result = catalog(**query)
        if len(result) == 0:
            return []
        return result[0].getObject()



from zope import schema
from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.memoize.compress import xhtml_compress
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize

# Quills imports
from quills.core.interfaces import IBaseContent
from quills.core.interfaces import IWeblogEnhanced
from quills.core.interfaces import IWeblog
from quills.app.utilities import recurseToInterface
from quills.app import QuillsAppMessageFactory as _

# Local imports
from base import BasePortletRenderer


PORTLET_TITLE = _(u"Weblog Authors")
PORTLET_DESC = _(u"This portlet lists weblog authors.")


class IWeblogAuthorsPortlet(IPortletDataProvider):

    show_location = schema.Bool(
        title=_(u'Show author location?'),
        description=_(u"If available, should the author's location be shown?"),
        required=True,
        default=True)

    show_portrait = schema.Bool(
        title=_(u'Show author portrait?'),
        description=_(u"If available, should the author's portrait be shown?"),
        required=True,
        default=True)

    show_description = schema.Bool(
        title=_(u'Show author description?'),
        description=_(u"If available, should the author's description be shown?"),
        required=True,
        default=True)


class Assignment(base.Assignment):

    implements(IWeblogAuthorsPortlet)

    def __init__(self,
                 show_location=True,
                 show_portrait=True,
                 show_description=True):
        self.show_location = show_location
        self.show_portrait = show_portrait
        self.show_description = show_description

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer):

    _template = ViewPageTemplateFile('authors.pt')

    @property
    def title(self):
        return PORTLET_TITLE
    
    @property
    def authors(self):
        weblog = self.getWeblog()
        return weblog.getAuthors()

    def getPortraitFor(self, author):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.getPersonalPortrait(author.getId())

    def getInfoFor(self, author):
        mtool = getToolByName(self.context, 'portal_membership')
        info = mtool.getMemberInfo(author.getId())
        if info is None:
            info = { 'fullname'    : author.getId(),
                     'description' : '',
                     'location'    : '',
                     'language'    : '',
                     'home_page'   : '',
                     'username'    : author.getId(),
                   }
        if not info['fullname']:
            info['fullname'] = author.getId()
        if not info['username']:
            info['username'] = author.getId()
        return info
    
    def getAuthorURL(self, author_id):
        weblog = self.getWeblogContentObject()
        return "%s/authors/%s" % (weblog.absolute_url(), author_id)
    
    def getAuthorsURL(self):
        weblog = self.getWeblogContentObject()
        return "%s/authors" % weblog.absolute_url()


class AddForm(base.AddForm):
    form_fields = form.Fields(IWeblogAuthorsPortlet)
    label = _(u'add-portlet', default=u"Add ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self, data):
        return Assignment(show_location=data.get('show_location', True),
                          show_portrait=data.get('show_portrait', True),
                          show_description=data.get('show_description', True))


class EditForm(base.EditForm):
    form_fields = form.Fields(IWeblogAuthorsPortlet)
    label = _(u'edit-portlet', default=u"Edit ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC



from Products.Five.testbrowser import Browser as BaseBrowser

# Import PloneTestCase - this registers more products with Zope as a side effect
from Products.PloneTestCase import PloneTestCase
from Products.PloneTestCase.PloneTestCase import setupPloneSite


class Browser(BaseBrowser):

    def addAuthorizationHeader(self,
                               user=PloneTestCase.default_user,
                               password=PloneTestCase.default_password):
        """Add an authorization header using the given or default credentials.
        """
        self.addHeader('Authorization', 'Basic %s:%s' % (user, password))
        return self


class BrowserMixin:
    """A mixin class that makes it possible to get hold of a `browser' from
    tests.
    """

    def getBrowser(self, logged_in=False):
        """Instantiate and return a testbrowser for convenience.
        """
        browser = Browser()
        if logged_in:
            browser.addAuthorizationHeader()
        return browser



# Zope imports
from zope.interface import implements

# Plone imports
from Products.CMFPlone.PloneBatch import Batch as PloneBatch
from Products.CMFCore.utils import getToolByName
from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.viewlets.common import TitleViewlet as BaseTitleViewlet

# Quills imports
from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogEntry
from quills.core.browser.interfaces import IWeblogView
from quills.core.browser.interfaces import IWeblogEntryView
from quills.core.browser.interfaces import ITopicView
from baseview import BaseView
from quills.app.interfaces import IWeblogEnhancedConfiguration
from quills.core.interfaces import IWeblogLocator
from quills.app import QuillsAppMessageFactory as _

try:
    from Products.CMFPlone.utils import translate
except ImportError:
    translate = None


class WeblogView(BaseView):
    """A class with helper methods for use in views/templates.

    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogView, WeblogView)
    True
    """

    implements(IWeblogView)

    def getWeblog(self):
        return IWeblog(self.context)

    def getWeblogContentObject(self):
        return self.context

    def getConfig(self):
        """See IWeblogView.
        """
        return IWeblogEnhancedConfiguration(self.getWeblogContentObject())

    def getWeblogEntriesDates(self, entries_dict):
        """See IWeblogView.
        """
        days = entries_dict.keys()
        days.sort()
        days.reverse()
        return days

    def sortWeblogEntriesToDates(self, lazy_entries, resolution='day'):
        """See IWeblogView.
        """
        if resolution == 'day':
            format = '%Y/%m/%d 00:00:00'
        elif resolution == 'month':
            format = '%Y-%m'
        elif resolution == 'year':
            format = '%Y'
        else:
            msg = "The 'resolution' parameter must be one of 'day', 'month', \
                   or 'year'.  You passed %s."
            raise Exception(msg % resolution)
        if isinstance(lazy_entries, PloneBatch):
            start = lazy_entries.start - 1
            end = lazy_entries.end
            lazy_entries = lazy_entries._sequence[start:end]
        results = {}
        for lazy_entry in lazy_entries:
            date = lazy_entry.getPublicationDate().strftime(format)
            try:
                if results[date]:
                    # Add the entry to the top of the list for that day
                    results[date].append(lazy_entry)
            except:
                results[date] = [lazy_entry,]
        return results


class WeblogEntryView(BaseView):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogEntryView, WeblogEntryView)
    True
    """

    implements(IWeblogEntryView, IViewView)

    def getWeblog(self):
        return self.getWeblogEntry().getWeblog()

    def getWeblogContentObject(self):
        return IWeblogEntry(self.context).getWeblogContentObject()

    def getWeblogEntry(self):
        return IWeblogEntry(self.context)

    def getConfig(self):
        """See IWeblogView.
        """
        weblog = self.getWeblogContentObject()
        return IWeblogEnhancedConfiguration(weblog)
    
    def workflow_state(self):
        """returns the current workflow state of the context"""
        wftool = getToolByName(self.context, 'portal_workflow')
        return wftool.getInfoFor(self.context, "review_state")


class TopicView(WeblogView):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(ITopicView, TopicView)
    True
    """

    implements(ITopicView)

    def getWeblog(self):
        """Return the weblog object.
        """
        locator = IWeblogLocator(self.context)
        return locator.find()

    def getWeblogContentObject(self):
        """ Return the content object underlying the weblog object,
        or the weblog object itself, if there is no such object.
        """
        weblog = self.getWeblog()
        return getattr(weblog, 'context', weblog)

    def getConfig(self):
        """See IWeblogView.
        """
        return IWeblogEnhancedConfiguration(self.getWeblogContentObject())

    def getLastModified(self, topic=None):
        """See ITopicView.
        """
        if topic is None:
            topic = self.context
        entries = topic.getEntries()
        if entries:
            # XXX modified should be in an interface
            return entries[0].modified

    def topicViewURLof(self, topic):
        """Return the URL of the view for given topic."""
        if self.viewContainerName():
            return '%s/%s/%s' % (self.getWeblogContentObject().absolute_url(),
                                 self.viewContainerName(),
                                 topic.getId())
        else:
            return '%s/%s' % (self.getWeblogContentObject().absolute_url(),
                                 topic.getId())            

    def viewContainerName(self):
        """Return the vitual container name that can be appended to the
        weblog url for browsing topics of a certain kind.
        """
        raise AttributeError("Sub-class responsibility")


class KeywordTopicView(TopicView):
    """A view for keyword topics."""

    def viewContainerName(self):
        """See base-class."""
        return "topics"


class AuthorTopicView(TopicView):
    """A view for author topics."""

    def viewContainerName(self):
        """See base-class."""
        return "authors"
    
class WeblogArchiveView(TopicView):
    """A class with helper methods for use in views/templates.
    """

    #implements(IWeblogArchiveView)
    pass


class TopicTitleViewlet(BaseTitleViewlet):
    """Render the title attribute of the HTML head in a meaningful and
    bookmark-friendly way. This one is for ITopic (keyword) content.
    """
    
    def update(self):
        """Customize plone.app.layout.viewlets.common.TitleViewlet. Look there
        for what goes on here."""
        context = self.context
        def blog_title():
            return context.getWeblog().getTitle()
        def post_title():
            message = _(u'posts_by_keywords', default=u'Posts about $keywords',
                        mapping={'keywords':context.getTitle()})
            if translate:
                ## Plone 4
                return translate(message)
            
            ## for Plone 3
            return context.translate(message)
         
        #BBB
        self.portal_title = blog_title
        #self.page_title = post_title        # does not work in Plone 4.3
        ## Plone 4
        self.site_title = blog_title

class AuthorTopicTitleViewlet(BaseTitleViewlet):
    """Render the title attribute of the HTML head in a meaningful and
    bookmark-friendly way. This one is for IAuthorTopic content.
    """
    
    def update(self):
        """Customize plone.app.layout.viewlets.common.TitleViewlet. Look there
        for what goes on here."""
        context = self.context
        def blog_title():
            return context.getWeblog().getTitle()
        def post_title():
            message = _(u'posts_by_authors', default=u'Posts by $authors',
                        mapping={'authors':context.getTitle()})
            if translate:
                ## Plone 4
                return translate(message)
            
            ## for Plone 3
            return context.translate(message)
        
        ##BBB
        self.portal_title = blog_title
        #self.page_title = post_title   # does not work in Plone 4.3
        ## Plone 4
        self.site_title = blog_title



# Zope imports
from zope.component import adapts
from zope.interface import implements

# Vice imports
from plone.syndication.outbound.interfaces import IFeed, IFeedItem
from plone.app.syndication.outbound.adapters.atct import ATFeedItemBase, ATFolderFeed

# Quills imports
from quills.core.interfaces import IWeblog, IWeblogEntry


class WeblogFeed(ATFolderFeed):
    """Adapter from IWeblog to IFeed.
    """

    implements(IFeed)
    adapts(IWeblog)


class WeblogEntryFeedItem(ATFeedItemBase):
    """Adapter from IWeblogEntry to IFeedItem
    """

    implements(IFeedItem)
    adapts(IWeblogEntry)

    @property
    def effective(self):
        """See IFeedItem
        """
        return self.context.effective()

    @property
    def tags(self):
        """See IFeedItem
        """
        return self.context.getTopics()

    @property
    def XHTML(self):
        """See IFeedItem
        """
        return self.context.getText()



# Zope imports
from zope.interface import implements

# Quills imports
from utilities import QuillsMixin
from utilities import recurseToInterface
from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogEnhanced
from quills.core.interfaces import IWeblogEntry
# Commented out to avoid circular import problems with archive and topic
# modules. The imports happen within each of the methods that require them.
#from topic import Topic, AuthorTopic

# XXX: We do need to add security assertions to this class because some
#      page templates access it even though they have been given no permission
#      to do so. Those templates should be fixed.
#      Only those methods actually accessed by restricted code are protected,
#      all other lack assertion and will deny access from restricted code.
#      Fixing this might even make acquistion unneccessary here.
from Acquisition import Implicit
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.CMFCore import permissions as permissions

# Renamed from WeblogEntryCatalogBrain to make all previous calls break.
# That way none will slip.
class CatalogBrainToWeblogEntry(QuillsMixin, Implicit):
    """Adapt a catalog brain to IWeblogEntry..

    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogEntry, CatalogBrainToWeblogEntry)
    True
    """

    security = ClassSecurityInfo()

    implements(IWeblogEntry)

    def __init__(self, brain):
        self.context = brain

    def getId(self):
        """See IWeblogEntry.
        """
        return self.context['id']

    def getTitle(self):
        """See IWeblogEntry.
        """
        return self.context['Title']

    def getTopics(self):
        """See IWeblogEntry.
        """
        from topic import Topic
        subjects = self.context['Subject']
        weblog_content = self.getWeblogContentObject()
        return [Topic(each).__of__(weblog_content) for each in subjects]

    security.declareProtected(permissions.View, "getAuthors")
    def getAuthors(self):
        """See IWeblogEntry.
        """
        from topic import AuthorTopic
        creators = self.context['listCreators']
        weblog_content = self.getWeblogContentObject()
        return [AuthorTopic(each).__of__(weblog_content) for each in creators]

    def getExcerpt(self):
        """See IWeblogEntry.
        """
        return self.context['Description']

    security.declareProtected(permissions.View, "getText")
    def getText(self):
        """See IWeblogEntry.
        """
        return self.context.getObject().getText()

    security.declareProtected(permissions.View, "getMimeType")
    def getMimeType(self):
        """See IWeblogEntry.
        """
        # In order to keep this implementation generic, we adapt the real object
        # to IWeblogEntry and just defer to that getMimeType implementation. The
        # reason being that we don't know if we're in a Quills world or a
        # QuillsEnabled world.
        return IWeblogEntry(self.context.getObject()).getMimeType()

    def getWeblogContentObject(self):
        """See IWeblogEntry.
        """
        return recurseToInterface(self.context.getObject(),
                                  (IWeblog, IWeblogEnhanced))
        
    def getWeblogEntryContentObject(self):
        """See IWeblogEntry
        """
        return self.context.getObject()

    def _getWeblogEntry(self):
        return IWeblogEntry(self.context.getObject())

    def getPublicationDate(self):
        """See IWeblogEntry.
        """
        return self.context['effective']

    def setTitle(self, title):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().setTitle(title)

    def setTopics(self, topic_ids):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().setTopics(topic_ids)

    def setExcerpt(self, excerpt):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().setExcerpt(excerpt)

    def setText(self, text, mimetype=None):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().setText(text, mimetype=mimetype)

    def edit(self, title, excerpt, text, topics, mimetype=None):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().edit(title, excerpt, text,
                                            topics, mimetype=mimetype)

    def setPublicationDate(self, datetime):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().setPublicationDate(datetime)

    def publish(self, pubdate=None):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().publish(pubdate)

    def retract(self):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().retract()

    def isPublished(self):
        """See IWeblogEntry.
        """
        self._getWeblogEntry().isPublished()

InitializeClass(CatalogBrainToWeblogEntry)



from zope.component import getMultiAdapter
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.PythonScripts.standard import url_quote_plus
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import transaction_note

from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogEntry


class DiscussionReply(BrowserView):
    """
    """

    def __call__(self,
                 subject,
                 body_text,
                 text_format='plain',
                 username=None,
                 password=None):
        """This method is lifted almost directly from CMFPlone's
        discussion_reply.cpy skin script. Modifications start at the point where
        we try to adapt to IWeblogEntry.
        """
        req = self.request
        if username or password:
            # The user username/password inputs on on the comment form were used,
            # which might happen when anonymous commenting is enabled. If they typed
            # something in to either of the inputs, we send them to 'logged_in'.
            # 'logged_in' will redirect them back to this script if authentication
            # succeeds with a query string which will post the message appropriately
            # and show them the result.  if 'logged_in' fails, the user will be
            # presented with the stock login failure page.  This all depends
            # heavily on cookiecrumbler, but I believe that is a Plone requirement.
            came_from = '%s?subject=%s&amp;body_text=%s' % (req['URL'], subject, body_text)
            came_from = url_quote_plus(came_from)
            portal_url = self.context.portal_url()
            return req.RESPONSE.redirect(
                '%s/logged_in?__ac_name=%s'
                '&amp;__ac_password=%s'
                '&amp;came_from=%s' % (portal_url,
                                       url_quote_plus(username),
                                       url_quote_plus(password),
                                       came_from,
                                       )
                )
        # if (the user is already logged in) or (if anonymous commenting is enabled and
        # they posted without typing a username or password into the form), we do
        # the following
        mtool = getToolByName(self.context, 'portal_membership')
        creator = mtool.getAuthenticatedMember().getId()
        dtool = getToolByName(self.context, 'portal_discussion')
        tb = dtool.getDiscussionFor(self.context)
        id = tb.createReply(title=subject, text=body_text, Creator=creator)
        reply = tb.getReply(id)
        # TODO THIS NEEDS TO GO AWAY!
        if hasattr(dtool.aq_explicit, 'cookReply'):
            dtool.cookReply(reply, text_format='plain')
        parent = tb.aq_parent
        # return to the discussable object.
        obj = self.context.plone_utils.getDiscussionThread(tb)[0]
        try:
            entry = IWeblogEntry(obj).__of__(self.context.aq_inner.aq_parent)
            # Check for the existence of a parent weblog to see if `obj' should
            # be treated as having an archive url.
            if IWeblog.providedBy(entry.getWeblog()):
                weview = getMultiAdapter((obj, self.request),
                                         name=u'weblogentry_view')
                base = weview.getArchiveURLFor(entry)
        except TypeError:
            base = obj.getTypeInfo().getActionInfo('object/view',
                                                               obj)['url']
        anchor = reply.getId()
        from Products.CMFPlone.utils import transaction_note
        transaction_note('Added comment to %s at %s' % (parent.title_or_id(),
                                                        reply.absolute_url()))
        self.context.plone_utils.addPortalMessage(_(u'Comment added.'))
        target = '%s#%s' % (base, anchor)
        return req.RESPONSE.redirect(target)



# Zope imports
from zope.interface import Interface
from zope import schema
from zope.i18nmessageid import MessageFactory

# quills imports
from quills.core.interfaces import IWeblogConfiguration
from quills.app import QuillsAppMessageFactory as _

class ITransientTopicContainer(Interface):
    """A marker interface that allows us to distinguish between IWeblog, which
    subclasses ITopicContainer, and the transient ITopicContainer implementation
    for the purposes of avoiding maximum recursion errors in the portlets
    machinery.
    """


class ITransientAuthorContainer(Interface):
    """A marker interface that allows us to distinguish between IWeblog, which
    subclasses IAuthorContainer, and the transient IAuthorContainer
    implementation for the purposes of avoiding maximum recursion errors in the
    portlets machinery.
    """

class ITransientArchive(Interface):
    """A marker interface that allows us to distinguish between IWeblog, which
    subclasses IWeblogArchive, and the transient IWeblogArchive implementation
    for the purposes of avoiding maximum recursion errors in the portlets
    machinery.
    """


class IDefaultTypeAwareWeblogConfiguration(IWeblogConfiguration):
    """
    """

    default_type = schema.TextLine(
        title=_(u'Default type.'),
        description=_(u'The default portal_type to add for this weblog.'),
        default=u'Document',
        required=False,
        )


class IStateAwareWeblogConfiguration(IWeblogConfiguration):
    """
    """

    published_states = schema.List(
        title=_(u'label_workflow_states_published', default=u'Published workflow states'),
        description=_(u'Workflow states to treat as published.'),
        default=[u'published'],
        required=True,
        value_type=schema.TextLine(),
        )

    draft_states = schema.List(
        title=_(u'label_workflow_states_draft', default=u'Draft workflow states'),
        description=_(u'Workflow states to treat as draft.'),
        default=[u'private'],
        required=True,
        value_type=schema.TextLine(),
        )


class IWeblogEnhancedConfiguration(IDefaultTypeAwareWeblogConfiguration,
                                   IStateAwareWeblogConfiguration):
    """
    """
    pass



from Acquisition import aq_base, aq_inner, aq_parent
from Products.CMFPlone.browser.interfaces import INavigationBreadcrumbs
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from quills.core.interfaces import IBaseContent, IWeblogEntry, IWeblogEnhanced
from zope.component import getMultiAdapter
from zope.interface import implements


class ArchiveAwareBreadcrumbs(object):
    """Workaround ATContentTypes' habit to use the inner acquisition chain
    for building the breadcrumbs. This would make the archive path disapear.
    """

    implements(INavigationBreadcrumbs)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def breadcrumbs(self):
        """This method should return a tuple of the form:

            ({'absolute_url': url_value,
                     'Title': title_value,
             },)
        """
        base = getMultiAdapter((self.container(),
                                self.request), name='breadcrumbs_view')
        crumbs = tuple(base.breadcrumbs())
        crumbs += ({'absolute_url': self.context.absolute_url(),
                   'Title': IWeblogEntry(self.context).getTitle(),
                   },)
        return crumbs

    def container(self):
        """Return the container of the WeblogEntry the breadcrumbs lead to.
        
        The previous implemention tried to skip over unwanted objects in the
        parent stack. Now and then something new turned up and broke the
        breadcrumbs (e.g. issue #179). This implemention will look for
        something we know. This is still a bad hack, but more reliable and
        a bit easier to grasp than the former (so I hope).
        """
        parent = aq_parent(aq_inner(self.context))
        while not IPloneSiteRoot.providedBy(parent):
            if ((IBaseContent.providedBy(parent) or
                  IWeblogEnhanced.providedBy(parent))
                and aq_base(parent) != aq_base(self.context)):
                return parent
            parent = aq_parent(aq_inner(parent))
        return parent



# zope imports
from zope.annotation.interfaces import IAnnotations
from BTrees.OOBTree import OOBTree

# Quills imports
from quills.app.portlets import tagcloud
from quills.app.portlets import weblogadmin
from quills.app.portlets import authors
from quills.app.portlets import recententries
from quills.app.portlets import quillslinks
from quills.app.portlets import recentcomments
from quills.app.portlets import archive

# plone imports
from plone.portlets.constants import CONTEXT_ASSIGNMENT_KEY, CONTEXT_CATEGORY
from plone.app.portlets.storage import PortletAssignmentMapping


DEFAULT_LEFT_PORTLETS = (
    ('tagcloud', tagcloud.Assignment, {}),
    ('archive', archive.Assignment, {}),
    ('quillslinks', quillslinks.Assignment, {}),
    )
DEFAULT_RIGHT_PORTLETS = (
    ('weblogadmin', weblogadmin.Assignment, {}),
    ('recententries', recententries.Assignment, {}),
    ('recentcomments', recentcomments.Assignment, {}),
    ('authors', authors.Assignment, {}),
    )


def registerContextPortlets(obj, event):
    """ An event listener that creates portlets assignments on context on blog creation
    """
    annotated = IAnnotations(obj)
    portlets = annotated.get(CONTEXT_ASSIGNMENT_KEY, OOBTree())

    left_portlets = portlets.get('plone.leftcolumn', PortletAssignmentMapping())
    right_portlets = portlets.get('plone.rightcolumn', PortletAssignmentMapping())

    for name, assignment, kwargs in DEFAULT_LEFT_PORTLETS:
        if not left_portlets.has_key(name):
            left_portlets[name] = assignment(**kwargs)
    for name, assignment, kwargs in DEFAULT_RIGHT_PORTLETS:
        if not right_portlets.has_key(name):
            right_portlets[name] = assignment(**kwargs)

    portlets['plone.leftcolumn'] = left_portlets
    portlets['plone.rightcolumn'] = right_portlets
    annotated[CONTEXT_ASSIGNMENT_KEY] = portlets





# Plone imports
from Products.CMFCore.utils import getToolByName

def setup_gs_profiles(portal, profiles, out):
    setup_tool = getToolByName(portal, 'portal_setup')
    for extension_id in profiles:
        try:
            setup_tool.runAllImportStepsFromProfile('profile-%s' % extension_id)
        except Exception, e:
            print >> out, "Error while trying to GS import %s (%s, %s)" \
                          % (extension_id, repr(e), str(e))
            raise

def addNewDiscussionReplyFormAction(portal, out=None):
    # We use this to stop discussion_reply_form from ending up redirecting to
    # a URL that doesn't take account of the weblog archive.
    # See quills.app.browser.discussionreply for more details.
    pfc = getToolByName(portal, 'portal_form_controller')
    pfc.addFormAction(object_id='discussion_reply_form',
                      status='success',
                      context_type='',
                      button='',
                      action_type='traverse_to',
                      action_arg='string:discussion_reply_redirect')
    if out is not None:
        print >> out, "Added form controller action to over-ride discussion_reply."

def delNewDiscussionReplyFormAction(portal, out=None):
    pfc = getToolByName(portal, 'portal_form_controller')
    # PFC has a terrible API for deleting which is clearly only intended to
    # happen ttw. This code is adapted from pfc._delFormActions.
    from Products.CMFFormController.FormAction import FormActionKey
    try:
        pfc.actions.delete(FormActionKey(object_id='discussion_reply_form',
                                         status='success',
                                         context_type='',
                                         button='',
                                         controller=pfc)
                          )
        msg = 'Removed form controller action that over-rode discussion_reply.'
    except KeyError:
        # The action wasn't found so it must have been removed already.
        msg = 'Did not find (for removal) form controller action that over-rode discussion_reply.'
    if out is not None:
        print >> out, msg



#
# Authors: Maurits van Rees <m.van.rees@zestsoftware.nl>
#
# Copyright 2006, Maurits van Rees
#
# Adapted from topic.py.
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

# Zope imports
from zope.interface import implements
from zope.component.interface import interfaceToName
from Acquisition import Implicit
from DateTime.DateTime import DateTime
from DateTime.DateTime import DateError
from OFS.Traversable import Traversable

# CMF imports
from Products.CMFCore.utils import getToolByName

# Plone imports
from Products.CMFPlone import PloneLocalesMessageFactory as _PLMF
from Products.CMFPlone.i18nl10n import monthname_msgid, monthname_english

# Quills imports
from quills.core.interfaces import IWeblogEntry
from quills.core.interfaces import IPossibleWeblogEntry
from quills.core.interfaces import IWeblogArchive
from quills.core.interfaces import IWeblogArchiveContainer
from acquiringactions import AcquiringActionProvider
from utilities import BloggifiedCatalogResults
from utilities import EvilAATUSHack
from utilities import QuillsMixin
from interfaces import ITransientArchive
from interfaces import IWeblogEnhancedConfiguration

# Check for Plone 4.0 or above
try:
    from Products.CMFPlone.factory import _IMREALLYPLONE4
except ImportError:
    PLONE40 = 0
else:
    PLONE40 = 1
    
class BaseArchive(QuillsMixin, AcquiringActionProvider, Traversable, Implicit):
    """Implementation of IWeblogArchive.
    """

    if PLONE40:
        __allow_access_to_unprotected_subobjects__ = True
    else:
        __allow_access_to_unprotected_subobjects__ = EvilAATUSHack()

    def __init__(self, *args, **kwargs):
        self._results = None

    def Description(self):
        """
        """
        return 'Archived weblog posts.'

    def __len__(self):
        """See IWeblogArchive.
        """
        return len(self.getEntries())


class ArchiveContainer(BaseArchive):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogArchive, ArchiveContainer)
    True
    """

    implements(IWeblogArchiveContainer)

    def __init__(self, id):
        self.id = str(id)
        self._years = None
        super(ArchiveContainer, self).__init__(self, id)

    def getId(self):
        """
        """
        return self.id

    def Title(self):
        """
        """
        return self.getId()

    def _getEntryYears(self):
        """
        """
        if self._years is None:
            years= {}
            for entry in self.getEntries():
                year = entry.getPublicationDate().strftime('%Y')
                years[year] = year
            years = years.keys()
            years.sort()
            self._years = years
        return self._years

    def getSubArchives(self):
        """
        """
        years = self._getEntryYears()
        return [YearArchive(year).__of__(self) for year in years]

    def getEntries(self, maximum=None, offset=0):
        """
        """
        # Just return the weblog's getEntries.
        self._results = self.getWeblog().getEntries()[offset:]
        if maximum is not None:
            self._results = self._results[:maximum]
        return self._results


class BaseDateArchive(BaseArchive):

    implements(IWeblogArchive, ITransientArchive)

    def Title(self):
        """
        """
        return self.getId()

    def getEntries(self, maximum=None, offset=0):
        """
        """
        if self._results is None:
            min_datetime, max_datetime = self._getDateRange()
            catalog = getToolByName(self, 'portal_catalog')
            weblog = self.getWeblogContentObject()
            weblog_config = IWeblogEnhancedConfiguration(weblog)
            path = '/'.join(weblog.getPhysicalPath())
            ifaces = [interfaceToName(catalog.aq_parent, IWeblogEntry),
                      interfaceToName(catalog.aq_parent, IPossibleWeblogEntry)]
            results = catalog(
                object_provides={'query' : ifaces, 'operator' : 'or'},
                path={'query':path, 'level': 0},
                review_state=weblog_config.published_states,
                effective={
                     'query' : [min_datetime, max_datetime],
                     'range': 'minmax'}
                )
            results = results[offset:]
            if maximum is not None:
                results = results[:maximum]
            self._results = BloggifiedCatalogResults(results)
        return self._results


class YearArchive(BaseDateArchive):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogArchive, YearArchive)
    True
    """

    def __init__(self, year):
        # Check year is of the right sort.
        int(year)
        self.year = year
        self._months = None
        super(BaseDateArchive, self).__init__(self, id)

    def getId(self):
        return str(self.year)

    def getTimeUnit(self):
        return 'Year'

    def _getEntryMonths(self):
        """
        """
        if self._months is None:
            months = {}
            for entry in self.getEntries():
                month = entry.getPublicationDate().strftime('%m')
                months[month] = month
            months = months.keys()
            months.sort()
            self._months = months
        return self._months

    def _getDateRange(self):
        min_datetime = DateTime('%s/01/01' % self.year)
        # +1 to max_datetime so that it becomes inclusive
        max_datetime = DateTime('%s/12/31' % self.year)+1
        return min_datetime, max_datetime

    def getSubArchives(self):
        """
        """
        months = self._getEntryMonths()
        return [MonthArchive(self.year, month).__of__(self) for month in months]


class MonthArchive(BaseDateArchive):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogArchive, MonthArchive)
    True
    """

    def __init__(self, year, month):
        int(year)
        int(month)
        self.year = year
        self.month = month
        self._days = None
        super(BaseDateArchive, self).__init__(self, id)

    def getId(self):
        return str(self.month)

    def getTimeUnit(self):
        return 'Month'

    def Title(self):
        # Get utranslate script from context...
        return getToolByName(self, 'utranslate')(msgid=monthname_msgid(self.month), default=monthname_english(self.month), domain='plonelocales').capitalize()

    def _getEntryDays(self):
        """
        """
        if self._days is None:
            days = {}
            for entry in self.getEntries():
                day = entry.getPublicationDate().strftime('%d')
                days[day] = day
            days = days.keys()
            days.sort()
            self._days = days
        return self._days

    def getSubArchives(self):
        """
        """
        days = self._getEntryDays()
        return [DayArchive(self.year, self.month, day).__of__(self) for day in days]

    def _getDateRange(self):
        # XXX Test me!  Need test to ensure that months with less than 31 days
        # are correctly handled.
        min_datetime = DateTime('%s/%s/01' % (self.year, self.month))
        day = 31
        while 1:
            try:
                max_datetime = DateTime('%s/%s/%s' % (self.year, self.month, day))
                break
            except DateError:
                day = day - 1
        # +1 to max_datetime so that it becomes inclusive
        return min_datetime, max_datetime+1


class DayArchive(BaseDateArchive):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogArchive, DayArchive)
    True
    """

    def __init__(self, year, month, day):
        int(year)
        int(month)
        int(day)
        self.year = year
        self.month = month
        self.day = day
        self._items = {}
        super(BaseDateArchive, self).__init__(self, id)

    def getId(self):
        return str(self.day)

    def getTimeUnit(self):
        return 'Day'

    def getSubArchives(self):
        """
        """
        []

    def _getDateRange(self):
        min_datetime = DateTime('%s/%s/%s' % (self.year, self.month, self.day))
        max_datetime = DateTime('%s/%s/%s' % (self.year, self.month, self.day))
        # +1 to max_datetime so that it becomes inclusive
        return min_datetime, max_datetime+1
    
    def __getitem__(self, key):
        if self._items.has_key(key):
            return self._items[key]
        min_datetime, max_datetime = self._getDateRange()
        catalog = getToolByName(self, 'portal_catalog')
        weblog = self.getWeblogContentObject()
        weblog_config = IWeblogEnhancedConfiguration(weblog)
        results = catalog(
                getId=key,
                review_state=weblog_config.published_states,
                effective={
                     'query' : [min_datetime, max_datetime],
                     'range': 'minmax'}
                )
        if len(results) != 1:
            # We can't find a suitable weblog entry, so raise a KeyError. This
            # causes the publisher to resort to looking-up/acquiring views.
            raise KeyError
        obj_brain = results[0]
        obj = obj_brain.getObject()
        self._items[key] = obj
        return obj



# Zope imports
from zope.interface import implements
from zope.component import getMultiAdapter

# Plone imports
from plone.portlets.interfaces import IPortletRetriever

# Quills imports
from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogEnhanced
from quills.app.utilities import recurseToInterface


class AcquiringWeblogPortletRetriever(object):
    """This implementation just delegates to the acquisition parent's
    IPortletRetriever implementation.
    """

    implements(IPortletRetriever)

    def __init__(self, context, storage):
        self.context = context
        self.storage = storage

    def getPortlets(self):
        """See IPortletRetriever.
        """
        weblog = self._getWeblog()
        weblog_retriever = getMultiAdapter((weblog, self.storage),
                                           IPortletRetriever)
        return weblog_retriever.getPortlets()

    def _getWeblog(self):
        context = self.context.aq_inner
        return recurseToInterface(context, (IWeblog, IWeblogEnhanced))



import unittest

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
ptc.setupPloneSite()

import quills

class TestCase(ptc.PloneTestCase):
    class layer(PloneSite):
        @classmethod
        def setUp(cls):
            fiveconfigure.debug_mode = True
            zcml.load_config('configure.zcml',
                             quills.app)
            fiveconfigure.debug_mode = False
    

def test_suite():
    return unittest.TestSuite([

        # Unit tests
        #doctestunit.DocFileSuite(
        #    'README.txt', package='quills.app',
        #    setUp=testing.setUp, tearDown=testing.tearDown),

        #doctestunit.DocTestSuite(
        #    module='quills.app.mymodule',
        #    setUp=testing.setUp, tearDown=testing.tearDown),


        # Integration tests that use PloneTestCase
        #ztc.ZopeDocFileSuite(
        #    'README.txt', package='quills.app',
        #    test_class=TestCase),

        #ztc.FunctionalDocFileSuite(
        #    'browser.txt', package='quills.app',
        #    test_class=TestCase),
        
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')


# Zope imports
from zope.interface import implements

# CMF imports
from Products.CMFCore.utils import getToolByName

# Local imports
from quills.remoteblogging.interfaces import IUIDManager, IUserManager


USER_INFO = {
    'name'      : 'no name',
    'email'     : 'no email',
    'userid'    : 'no user id',
    'firstname' : 'no first name',
    'lastname'  : 'no last name',
    'url'       : 'no url',
    }


class WeblogUserManager:
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IUserManager, WeblogUserManager)
    True
    """

    implements(IUserManager)
    
    def __init__(self, context):
        self.context = context

    def getWeblogsForUser(self, user_id):
        """See IUserManager.
        """
        # This method returns a list with details for *only* the weblog that is
        # being adapted here.
        parent_blog = self.context
        blogs = []
        blogs.append(
            {'url': self.context.absolute_url(),
             'blogid' : IUIDManager(self.context).getUID(),
             'blogName' : self.context.Title(),
            }
            )
        return blogs

    def getUserInfo(self, user_id):
        """See IUserManager.
        """
        membership = getToolByName(self.context, 'portal_membership')
        info = USER_INFO.copy()
        member = membership.getAuthenticatedMember()
        if member:
            for key,value in info.items():
                info[key] = getattr(member, key, None) or value
        return info


class PortalUserManager:

    implements(IUserManager)

    def __init__(self, context):
        self.context = context

    def getWeblogsForUser(self, user_id):
        """See IUserManager.
        """
        # This method returns a list with details for all weblogs within the
        # portal.
        catalog = getToolByName(self.context, 'portal_catalog')
        results = catalog(meta_type='Weblog', Creator=user_id)
        blogs = []
        for item in results:
            obj = item.getObject()
            blogs.append(
                    {'url': obj.absolute_url(),
                     'blogid' : IUIDManager(obj).getUID(),
                     'blogName' : obj.Title()
                     }
                        )
        return blogs

    def getUserInfo(self, user_id):
        """See IUserManager.
        """
        membership = getToolByName(self.context, 'portal_membership')
        info = USER_INFO.copy()
        member = membership.getAuthenticatedMember()
        if member:
            for key,value in info.items():
                info[key] = getattr(member, key, None) or value
        return info



from Acquisition import aq_base
from ZPublisher.BaseRequest import DefaultPublishTraverse
from archive import ArchiveContainer
from archive import DayArchive
from archive import MonthArchive
from archive import YearArchive
from quills.core.interfaces import IAuthorContainer
from quills.core.interfaces import ITopicContainer
from quills.core.interfaces import IWeblog
from quills.core.interfaces import IWeblogArchive
from quills.core.interfaces import IWeblogConfiguration
from topic import AuthorContainer
from topic import AuthorTopic
from topic import Topic
from topic import TopicContainer
from zope.component import adapts, getMultiAdapter
from zope.interface import alsoProvides
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.publisher.interfaces.http import IHTTPRequest

try:
    from zope.app.publisher.browser import getDefaultViewName
except ImportError:
    from zope.publisher.defaultview import getDefaultViewName


class IInsideWeblog(IDefaultBrowserLayer):
    """Marker interface for Requests to signal that traversal moves
    inside a QuillsEnabled weblog. To trigger a view or a browser page
    only for content inside a weblog simply set this interface as the layer
    of the view/page.  
    """


class WeblogTraverser(DefaultPublishTraverse):
    """ """
    adapts(IWeblog, IHTTPRequest)
    
    def publishTraverse(self, request, name):
        # Mark request: we're in a weblog
        # You can trigger views/pages by using layer "IInsideWeblog" now.
        alsoProvides(request, IInsideWeblog)

        # Only intercept certain names...
        if name == 'topics':
            # XXX 'topics' should probably not be hard-coded here.  Rather,
            # it should be looked up on weblog config.
            return TopicContainer('topics').__of__(self.context)
        elif name == 'authors':
            # XXX 'authors' should probably not be hard-coded here.  Rather,
            # it should be looked up on weblog config.
            return AuthorContainer('authors').__of__(self.context)
        elif self.isArchiveFolder(name):
            return ArchiveContainer(name).__of__(self.context)
        elif self.isYear(name):
            return YearArchive(name).__of__(self.context)
        return super(WeblogTraverser, self).publishTraverse(request, name)

    def isArchiveFolder(self, name):
        """Test if 'name' is an archive folder.  This is True when name is the
        value indicated by weblog_config
        """
        weblog_config = IWeblogConfiguration(self.context)
        if name == weblog_config.archive_format:
            return True
        return False

    def isYear(self, name):
        try:
            year = int(name)
        except ValueError:
            return False
        else:
            from DateTime import DateTime
            try:
                tryDate = DateTime(year,1,1)
            except:
                # Bad year
                return False
            else:
                return True


class WeblogArchiveTraverser(DefaultPublishTraverse):
    """ """
    adapts(IWeblogArchive, IHTTPRequest)

    def publishTraverse(self, request, name):
        """
        """
        isdate = False
        try:
            int(name)
            isdate = True
        except ValueError:
            pass
        if not isdate:
            # We're at the end of the archive hierarchy, and now need to
            # traverse for the actual IWeblogEntry item.  The trick here is to
            # lookup the weblogentry-ish view on an object that is an
            # IPossibleWeblogEntry so that it gets rendered in the weblog-ish
            # way.
            # So, we do standard traversal to get the actual object.
            obj = super(WeblogArchiveTraverser,
                        self).publishTraverse(request, name)
            return obj

        year = getattr(aq_base(self.context), 'year', None)
        month = getattr(aq_base(self.context), 'month', None)
        day = getattr(aq_base(self.context), 'day', None)
        if day is not None:
            return super(WeblogArchiveTraverser,
                         self).publishTraverse(request, name)
        if month is not None:
            archive = DayArchive(year, month, name)
        elif year is not None:
            archive = MonthArchive(year, name)
        else:
            archive = YearArchive(name)
        ob = archive.__of__(self.context)
        return ob


class BaseContainerTraverser(DefaultPublishTraverse):
    """Base traverser for Topics and Authors
    """

    def publishTraverse(self, request, name):
        """Interpret any remaining names on the traversal stack as keywords
        of this topic container. Only exception is an appended view name at
        the end of the traversal stack, or if a sub-class aborts the process

        This operation will empty the traversal stack! While the view have
        an instance of ``klass`` as context, it will have the topic container
        as acquistion parent! The ``klass`` instance will be initialized with
        the keywords taken from the traversal stack.
        """
        self.request = request

        # collect keywords
        keywords = []
        while self.hasMoreNames():
            keywords.append(name)
            name = self.nextName()

        # Determine view
        view_name = None
        context = None
        if name.startswith('@@'): 
            # A view has explicitly been requested, so make that the 
            # view_name (stripping off the leading @@ which causes view 
            # lookups to fail otherwise). 
            view_name = name[2:]
        else:
            # last name is another keyword
            keywords.append(name)

        if len(keywords) == 0:
            # No keywords given, just a view specified. Create a view for the
            # topic container.
            context = self.context
        else:
            context = self.wrapUp(keywords).__of__(self.context)

        view_name = view_name or getDefaultViewName(context, request)
        view = getMultiAdapter((context, request), name=view_name) 
        return view.__of__(self.context) 

    def nextName(self): 
        """Pop the next name off of the traversal stack.
        """
        return self.request['TraversalRequestNameStack'].pop() 

    def hasMoreNames(self):
        """Are there names left for traversal?
        """
        return len(self.request['TraversalRequestNameStack']) > 0

    def wrapUp(self, keywords): 
        """Create a content object for the given keywords. 
         """ 
        raise NotImplementedError("subclass responsibility") 


class TopicContainerTraverser(BaseContainerTraverser):
    """Topic container traversal
    
    blog/topics/topic_name
    """
    adapts(ITopicContainer, IHTTPRequest)

    def wrapUp(self, keywords): 
        """See super-class.""" 
        return Topic(keywords)


class AuthorContainerTraverser(BaseContainerTraverser):
    """Author container traversal
    
    blog/authors/author_id
    """
    adapts(IAuthorContainer, IHTTPRequest)

    def wrapUp(self, keywords): 
        """See super-class.""" 
        return AuthorTopic(keywords)

            



from zope import schema
from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.memoize.compress import xhtml_compress
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

# Quills imports
from quills.core.interfaces import IWeblogLocator
from quills.app.browser.baseview import BaseView
from quills.app import QuillsAppMessageFactory as _

# Local imports
from base import BasePortletRenderer


PORTLET_TITLE = _(u"Recent Entries")
PORTLET_DESC = _(u"This portlet lists recent weblog entries.")


class IRecentWeblogEntriesPortlet(IPortletDataProvider):

    max_entries = schema.Int(
        title=_(u'Maximum entries'),
        description=_(u"What's the maximum number of entries to list?"),
        required=True,
        default=5)


class Assignment(base.Assignment):

    implements(IRecentWeblogEntriesPortlet)

    def __init__(self, max_entries=5):
        self.max_entries = max_entries

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer, BaseView):

    _template = ViewPageTemplateFile('recententries.pt')

    @property
    def title(self):
        return PORTLET_TITLE

    @property
    def getEntries(self):
        weblog = self.getWeblog()
        return weblog.getEntries(maximum=self.data.max_entries)


class AddForm(base.AddForm):
    form_fields = form.Fields(IRecentWeblogEntriesPortlet)
    label = _(u'add-portlet', default=u"Add ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self, data):
        return Assignment(max_entries=5)


class EditForm(base.EditForm):
    form_fields = form.Fields(IRecentWeblogEntriesPortlet)
    label = _(u'edit-portlet', default=u"Edit ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC



# zope imports
from zope.interface import implements

# CMF imports
from Products.CMFCore.utils import getToolByName

# Local imports
from quills.remoteblogging.interfaces import IUIDManager


class UIDManager:
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IUIDManager, UIDManager)
    True
    """

    implements(IUIDManager)

    def __init__(self, context):
        self.context = context

    def getByUID(self, uid):
        """See IUIDManager.
        """
        if uid=='0' or uid=='' or uid is None:
            return self.context
        uid_catalog = getToolByName(self.context, 'uid_catalog')
        lazy_cat = uid_catalog(UID=uid)
        if len(lazy_cat)>0:
            obj = lazy_cat[0].getObject()
            return obj
        else:
            return self.context

    def getUIDFor(self, obj):
        """See IUIDManager.
        """
        if obj is None:
            obj = self.context
        uid = getattr(obj, 'UID', None)
        if uid is None:
            # XXX A bit hacky! This is for the case where obj is an adapter
            # (i.e. in QuillsEnabled). It may make sense to refactor this so
            # that IWeblogEntry has a getUID method, although that seems rather
            # heavy-handed given that IWeblogEntry has no real reason to have
            # that method, other than to support remote blogging.
            uid = getattr(obj.context, 'UID')
        if callable(uid):
            return uid()
        return uid

    def getUID(self):
        """See IUIDManager.
        """
        return self.getUIDFor(self.context)



from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.app.portlets.browser.formhelper import NullAddForm
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from quills.app import QuillsAppMessageFactory as _
from base import BasePortletRenderer

PORTLET_TITLE = _(u"Tag Cloud")
PORTLET_DESC = _(u"This portlet displays a tag cloud.")


class ITagCloudPortlet(IPortletDataProvider):
    """A tag cloud portlet.
    """

class Assignment(base.Assignment):
    implements(ITagCloudPortlet)

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer):
    _template = ViewPageTemplateFile('tagcloud.pt')

    @property
    def title(self):
        return PORTLET_TITLE
    
    def getCloud(self):
        weblog = self.getWeblog()
        if weblog == []:
            return []
        # Get a list of topics, sorted alphabetically
        topics = weblog.getTopics()
        if not topics:
            return []
        # Create a list of entries for each topic
        topic_size = []
        for topic in topics:
            topic_size.append(len(topic))
        # Find the Min and Max number entries
        min_topic_size = min(topic_size)
        max_topic_size = max(topic_size)
        # Make sure size_topic_size is never 0, or we get ZeroDivisionError later
        size_topic_size = max_topic_size - min_topic_size + 0.0 or 1
        # Create a list to return tuples containing the:
        #    - topic object
        #    - relative weight of the topic, 0.0 to 1.0
        #    - number of entries for that topic, since we already calculated it.
        # Skip topics that have no entries.
        result = []
        for topic, number_of_entries in zip(topics, topic_size):
            if number_of_entries > 0:
                # Normalize the result
                rel = ( (number_of_entries - min_topic_size ) / size_topic_size )
                # Add the result
                cloud_dict = {'topic'               :   topic,
                              'rank'                :   rel,
                              'size'                :   int(rel*10),
                              'number_of_entries'   :   number_of_entries}
                result.append(cloud_dict)
        # Done...
        return result
    
    def getTopicsURL(self):
        weblog = self.getWeblog()
        if weblog == []:
            return ''
        # XXX This feels very hacky!
        if getattr(weblog, 'absolute_url', None):
           url = weblog.absolute_url()
        else:
            # The `weblog' object must be an adapter, so we'll use its context
            # to get hold of the url.
            url = weblog.context.absolute_url()
        return "%s/topics" % url


class AddForm(NullAddForm):
    form_fields = form.Fields(ITagCloudPortlet)
    label = _(u'add-portlet', 
            default=u"Add ${portlet-name} Portlet", 
            mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self):
        return Assignment()


class EditForm(base.EditForm):
    form_fields = form.Fields(ITagCloudPortlet)
    label = _(u'edit-portlet', 
            default=u"Edit ${portlet-name} Portlet", 
            mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC




from controllerView import FormControllerView, getToolByName
from quills.app.utilities import talkbackURL

class ManageCommentsView(FormControllerView):
    """ The view class for the comments management form """
    
    def __init__(self, context, request):
        super(ManageCommentsView, self).__init__(context, request)

        self.form_submitted = bool(request.get('form.submitted'))
        if bool(request.get('form.button.Delete')):
            self.mode = "delete"
        elif bool(request.get('form.button.Update')):
            self.mode = "update"
        elif bool(request.get('form.button.ResetFilter')):
            self.mode = "reset"
        else:
            self.mode = 'display'

        self.author = request.get('form.field.author', '')
        self.subject = request.get('form.field.subject', '')
        self.selected_comments = request.get('selected_comments', [])
        self.portal_catalog = getToolByName(self.context,'portal_catalog')

        self.contentFilter = {
            'portal_type' : 'Discussion Item', 
            'sort_on' : 'created', 
            'sort_order' : 'reverse',
            'path' : {'query' : '/'.join(self.context.getPhysicalPath()),}
        }

        self.filtered = False
        if self.mode in ['delete', 'display', 'update']:
            if self.author:
                self.contentFilter['Creator'] = self.author
                self.filtered = True
            if self.subject:
                self.contentFilter['Title'] = self.subject
                self.filtered = True
        if self.mode == "display":
            self.getComments()

    def validate(self):
        """ performs validation and returns an errors dictionary """
        errors = {}
        if self.mode=='delete':
            if self.selected_comments == []:
                errors['status'] = 'failure'    # errors must not be empty...            
                self.setMessage('You must select at least one comment for deletion.')
                self.getComments()
        return errors

    def control(self):
        """ performs the actions after a successful validation possibly
            returning an errors dictionary """

        if self.mode == "update":
            self.setMessage('Filter applied.')
        elif self.mode == "reset":
            self.setMessage('Filter reset.')
            self.author = None
            self.subject = None
        elif self.mode == "delete":
            discussion_tool = getToolByName(self.context, 'portal_discussion')
            for path in self.selected_comments:
                self.deleteReply(discussion_tool, self.portal_catalog, path)
            self.setMessage('%s comments have been deleted.' % str(len(self.selected_comments)))

        self.getComments()
        return {}
            
    def deleteReply(self, dtool, pcatalog, path):
        discussion_item = pcatalog(path=path)[0].getObject()
        obj = discussion_item.parentsInThread()[0]
        discussion = dtool.getDiscussionFor(obj)
        discussion.deleteReply(discussion_item.getId())

    def getComments(self):
        self.comment_brains = self.portal_catalog(self.contentFilter)
        self.num_of_comments = len(self.comment_brains)
        self.has_comments = self.num_of_comments > 0
        return self.comment_brains

    def talkbackURL(self, item):
        return talkbackURL(item)



from setuptools import setup, find_packages
import os

version = '1.8'

setup(name='quills.app',
      version=version,
      description="The Quills blogging suite. Contains code shared between "
                  "Products.Quills and  Products.QuillsEnabled.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone blogging',
      author='Quills Team',
      author_email='',
      url='https://github.com/collective/quills.app',
      download_url="https://github.com/collective/quills.app",
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['quills'],
      include_package_data=True,
      zip_safe=False,
      # Do not remove version constrains, see issue #196
      install_requires=[
          'setuptools',
          'quills.core>=1.7.0,<=1.7.99',
          'Products.basesyndication',
          'Products.fatsyndication>=1.0.0'
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



"""
Test install/uninstall of the product.

:Authors: - Jan Hackel <plonecode@hackel.name>

$Id$
"""

__docformat__ = "reStructuredText"

from Products.PloneTestCase import PloneTestCase
from Products.Five.testbrowser import Browser
from Products.CMFCore.utils import getToolByName
import unittest

PORTAL_URL="http://nohost/plone"

class TestInstall(PloneTestCase.FunctionalTestCase):
    """ Provide functionality needed across the test cases.
    """

    def productExtensionProfile(self):
        """Return the Generic Setup extension profile of the product
        to be tested.
        """
        raise NotImplementedError('Subclass responsibility')

    def productControlPanelLabel(self):
        """Return label by which the checkbox that selects the
        product to be tested in the Quickinstaller control panel.
        """
        raise NotImplementedError('Subclass responsibility')

    def productQuickInstallerName(self):
        """Return the name of the product to be tested. Must be fitting
        for the Quickinstaller API.
        """
        raise NotImplementedError('Subclass responsibility')

    def product_is_usable(self):
        """ Check if the product is usable.
        """
        raise NotImplementedError('Subclass responsibility')

    def afterSetUp(self):
        """Setup test enviroment. Will be executed before every test-case.
        """
        self.browser = Browser()
        self.browser.addHeader('Authorization',
                               'Basic %s:%s' % (PloneTestCase.default_user,
                                                PloneTestCase.default_password))
        self.login()
        self.setRoles(('Manager',))

        # When this test-suite is run together with the other test-cases
        # we might get a plone instance where Quills is already installed.
        # Now, having to uninstall this package in a test that is supposed
        # to test install/uninstall is rather doubious... Setting up
        # a Plone Site with a custom id would be a solution, but that
        # code is broken (Plone ticket #9078).
        quickInstaller = getToolByName(self.portal, 'portal_quickinstaller')
        if quickInstaller.isProductInstalled('Quills'):
            quickInstaller.uninstallProducts( ("Quills",) )

        if quickInstaller.isProductInstalled('Products.Quills'):
            quickInstaller.uninstallProducts( ("Products.Quills",) )

    def install_ttw(self):
        """Install the product with the Quickinstaller Control Panel.
        """
        cpLabel = self.productControlPanelLabel()
        productName = self.productQuickInstallerName()
        # Go to the control panel, select and install the product there.
        browser = self.browser
        browser.open(PORTAL_URL + '/prefs_install_products_form')
        form = browser.getForm(action=PORTAL_URL + 
                                   '/portal_quickinstaller/installProducts')
        form.getControl(label=cpLabel).selected = True

        # A bug in zope.testbrowser (Launchpad #98437) will cause "submit"
        # to throw a nasty looking exception, because the Quickinstaller
        # expectsto get a correct HTTP referrer, but the testbrowser always
        # sends'localhost'. It will be taken for a relative URL, where it is
        # pure nonsense. Once this bug is solved, the try-except-cause may 
        # safely go away.
        from urllib2 import HTTPError
        try:
            form.submit('Install')
        except HTTPError: 
            # redirect manually
            browser.open(PORTAL_URL + '/prefs_install_products_form')
        
        # Now the Control Panel should show it as installed.
        form = browser.getForm(action=PORTAL_URL +
    	                              '/portal_quickinstaller$')
        self.assertTrue(form.getControl(label=cpLabel) is not None)
        
        # And the Quickinstaller should report so too.
        quickInstaller = getToolByName(self.portal, 'portal_quickinstaller')
        self.assertTrue(quickInstaller.isProductInstalled(productName))

    def uninstall_ttw(self):
        """ Uninstall the product using the Quickinstaller control panel.
        """
        cpLabel = self.productControlPanelLabel()
        productName = self.productQuickInstallerName()
        # Go to the control panel, select and uninstall the product there.
        browser = self.browser
        browser.open(PORTAL_URL + '/prefs_install_products_form')
        form = browser.getForm(action=PORTAL_URL + 
                                   '/portal_quickinstaller$')
        form.getControl(label=cpLabel).selected = True
        
        # See install_ttw.
        from urllib2 import HTTPError
        try:
            form.submit('Uninstall')
        except HTTPError: 
            # redirect manually
            browser.open(PORTAL_URL + '/prefs_install_products_form')
        
        # Now the Control Panel should show it as installable.
        form = browser.getForm(action=PORTAL_URL + 
    	                              '/portal_quickinstaller/installProducts')
        self.assertTrue(form.getControl(label=cpLabel) is not None)
        
        # And the Quickinstaller should report it as not installed.
        quickInstaller = getToolByName(self.portal, 'portal_quickinstaller')
        self.assertTrue(not quickInstaller.isProductInstalled(productName))

    def install_by_quickinstaller_tool(self):
        """Install the product, using the quickinstaller API.
        """
        productName = self.productQuickInstallerName()
        quickInstaller = getToolByName(self.portal, 'portal_quickinstaller')
        self.assertTrue( not quickInstaller.isProductInstalled(productName))
        quickInstaller.installProduct(productName)
        self.assertTrue( quickInstaller.isProductInstalled(productName) )

    def uninstall_by_quickinstaller_tool(self):
        """Uninstall the product, using the quickinstaller API.
        """
        productName = self.productQuickInstallerName()
        quickInstaller = getToolByName(self.portal, 'portal_quickinstaller')
        self.assertTrue( quickInstaller.isProductInstalled(productName) )
        quickInstaller.uninstallProducts( (productName,) )
        self.assertTrue( not quickInstaller.isProductInstalled(productName) )

    def install_by_generic_setup(self):
        """Install the product, using a Generic Setup extensio profile.
        """
        extensionProfile = self.productExtensionProfile()
        gs = getToolByName(self.portal, 'portal_setup')
        self.assertEqual(gs.getBaselineContextID(),
                         "profile-Products.CMFPlone:plone")
        # gs.setBaselineContext("profile-Products.CMFPlone:plone")
        gs.runAllImportStepsFromProfile("profile-%s" % (extensionProfile,))

    def test_install_ttw(self):
        """ Test that the product installation actually gets us an usable
        product. Use the quickinstaller control panel web page for that.
        """
        self.install_ttw()
        self.product_is_usable()
        
    def test_install_by_quickinstaller_tool(self):
        """ Test that the product installation actually gets us an usable
        product. Uses the quickinstaller tool API.
        """
        self.install_by_quickinstaller_tool()
        self.product_is_usable()

    def test_install_by_generic_setup(self):
        """ Test that the product installation actually gets us an usable
        product. Uses a generic setup extension profile.
        """
        self.install_by_generic_setup()
        self.product_is_usable()

    def test_uninstall_ttw(self):
        """ Test that after uninstalling the product it can no longer be used.
        Use the quickinstaller control panel web page for that.
        """
        self.install_ttw()
        self.uninstall_ttw()
        # The portal type "Weblog" should no longer be available for adding.
        from mechanize import LinkNotFoundError
        self.browser.open(PORTAL_URL)
        self.assertRaises( LinkNotFoundError, self.browser.getLink,
                           id="weblog" )

    def test_uninstall_by_quickinstaller_tool(self):
        """ Test that after uninstalling the product it can no longer be used.
        Use the quickinstaller tool API.
        """
        self.install_by_quickinstaller_tool()
        self.uninstall_by_quickinstaller_tool()

        # The portal type "Weblog" should no longer be available for adding.
        from mechanize import LinkNotFoundError
        self.browser.open(PORTAL_URL)
        self.assertRaises( LinkNotFoundError, self.browser.getLink,
                           id="weblog" )



"""Provide access to Quills' metal macros."""

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class Macros(BrowserView):
    def __getitem__(self, key):
        return self.template.macros[key]


class HeaderMacros(Macros):
    template = ViewPageTemplateFile('quills_header_macros.pt')


class WeblogEntryMacros(Macros):
    template = ViewPageTemplateFile('quills_entry_macros.pt')


class WeblogMacros(Macros):
    template = ViewPageTemplateFile('quills_weblog_macros.pt')






from quills.app.browser.baseview import BaseView
from Products.CMFCore.utils import getToolByName

# A basic form controller designed for maximum compatibility with CMFFormController
# i.e. changes to the existing templates are minimised.
# n.b. this should probably be phased out again at some time by using formlib
# but for the time being it's rather useful to have.
# Written by Andreas Zeidler a.k.a. witsch based on the `FormControllerView` in membrane
# (http://dev.plone.org/collective/browser/membrane/trunk/browser/tool_zmi_views.py?rev=21197)

class FormControllerViewState:
    """ fake formcontroller's state class for page templates """
    
    def __init__(self, errors):
        self.errors = errors
    
    def getErrors(self, *args, **kw):

        return self.errors
    

class FormControllerView(BaseView):
    """ a very basic formcontroller-like view class """
    
    def __call__(self, *args, **kw):
        self.args = args
        self.kw = kw
        errors = {}
        if self.request.get('submitted', None) or self.request.get('form.submitted', None):
            errors = self.validate()
            if not errors:
                errors = self.control()
                if not errors:
                    return self.action(*args, **kw)
            elif type(errors) == type(''):      # a redirection was given...
                return self.callByName(errors)
        return self.callIndex(errors, *args, **kw)
    
    def validate(self):
        """ performs validation and returns an errors dictionary """
        return {}
    
    def control(self):
        """ performs the actions after a successful validation possibly
            returning an errors dictionary """
        return {}
    
    def action(self, *args, **kw):
        """ this can be called to return some output or redirect to another
            page after the control action was successfully called """
        return self.callIndex(*args, **kw)
    
    def translate(self, msg):
        ltool = getToolByName(self.context, 'portal_languages')
        lang = ltool.getPreferredLanguage()
        panel = getToolByName(self.context, 'Control_Panel')
        return panel.TranslationService.utranslate(msgid=msg,
            domain='quills', target_language=lang, context=self.context)
    
    def translateErrors(self, errors):
        translated = {}
        for field, msg in errors.items():
            if type(msg) == type(()):
                if len(msg) == 2:
                    msgstr, msgid = msg
                else:
                    msgstr, msgid = msg[0], msg[0]
                msg = self.translate(msgid)
                if msg == msgid:
                    msg = self.translate(msgstr)
            translated[field] = msg
        return translated
    
    def callIndex(self, errors={}, *args, **kw):
        """ call the original template with some additional parameters """
        errors = self.translateErrors(errors)
        return self.index(errors=errors, 
            state=FormControllerViewState(errors), *args, **kw)
            
    def callByName(self, name, errors={}):
        """ call the original template with some additional parameters """
        obj = getattr(self.context, name)
        errors = self.translateErrors(errors)
        return obj(errors=errors, 
            state=FormControllerViewState(errors), *self.args, **self.kw) 

    def setMessage(self, msg):
        self.request.set('portal_status_message', msg)
    
    def getToolByName(self, name):
        """ helper function to return the named portal tool """
        return getToolByName(self.context, name, None)
    
    def getFormVar(self, name):
        """ return the value of a request variable or None """
        var = self.request.get(name, None)
        if var is not None and type(var) == type(''):
            var = var.strip()
        return var



from plone.app.layout.globals.context import ContextState
try:
    from zope.app.publisher.browser import getDefaultViewName
except ImportError:
    from zope.publisher.defaultview import getDefaultViewName


class QuillsContextState(ContextState):
    """A custom view component that knows how to answer is_view_template for
    Quills objects. This is needed for Plone to recognize view templates at
    virtual URLSs, like for instance archive URLs. Plone's default
    implementation bases it's decision only on the absolute object
    URL (which differs from the virtual URL) and thus always yields "False".
    See Quills issues #97 and #193 for the ill effects this could have.
    """

    def is_view_template(self):
        # We only do something special for IWeblogEntry objects.
        #if not IWeblogEntry.providedBy(self.context):
        #    return super(QuillsContextState, self).is_view_template()
        url = self.current_base_url()
        url_parts = url.split('/')
        if url_parts[-1] == self.context.getId():
            # The last part of the URL is the same as the id of the object, so
            # we assume that there is no extra view specified, and thus that we
            # must be on the default view of the object.
            return True
        elif url_parts[-1] == getDefaultViewName(self.context, self.request):
            # The last part of the URL is the same as the default view name, so
            # we assume that we are using the default view.
            return True
        return False



# -*- coding: utf-8 -*-
""" 
XXX TODO
--------
    
- Topics containing multiple keywords do not work properly: 
  - Neither author nor keyword topics can return multiple
    images as would be required when filtering by multiple 
    keywords/authors --- jhackel
"""

# Standard library imports
from types import StringTypes

# Zope 3 imports
from zope.interface import implements
from zope.component import getUtility
from zope.component.interface import interfaceToName

# Zope 2 imports
from Acquisition import Implicit, aq_base
from OFS.Traversable import Traversable

# CMF imports
from Products.CMFCore.utils import getToolByName

# Plone imports
from plone.i18n.normalizer.interfaces import IIDNormalizer

# Quills imports
from quills.core.interfaces import ITopic
from quills.core.interfaces import IAuthorTopic
from quills.core.interfaces import ITopicContainer
from quills.core.interfaces import IAuthorContainer
from quills.core.interfaces import IWeblogEntry
from quills.core.interfaces import IPossibleWeblogEntry
from acquiringactions import AcquiringActionProvider
from utilities import BloggifiedCatalogResults
from utilities import EvilAATUSHack
from utilities import QuillsMixin
from interfaces import ITransientTopicContainer
from interfaces import ITransientAuthorContainer
from interfaces import IWeblogEnhancedConfiguration


class Topic(QuillsMixin, AcquiringActionProvider, Traversable, Implicit):
    """Implementation of ITopic as a transient wrapper around a keywords.

    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(ITopic, Topic)
    True
    """

    implements(ITopic)

    __allow_access_to_unprotected_subobjects__ = EvilAATUSHack()

    def __init__(self, keywords):
        if isinstance(keywords, StringTypes):
            self.keywords = [keywords,]
        else:
            self.keywords = keywords
        # We will cache catalog queries for performance reasons.
        # This should not cost anything (except a bit of RAM) as Topic instances
        # only last for the duration of a REQUEST anyway.
        self.results = None

    def getId(self):
        """See ITopic.
        """
        return self.keywords[0]

    def getKeywords(self):
        """See ITopic.
        """
        # Return a copy so they can't be changed by other code.
        return self.keywords[:]

    def getTitle(self):
        """See ITopic.
        """
        return ' & '.join(self.keywords).decode('utf-8')

    def getDescription(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.Description()
        return ''

    def getImage(self):
        """See ITopic.
        """
        # XXX: Behaves awkwardly for multiple keyword topics. The
        # keyword will determine the image.

        # N.B. 'topic_images' is acquired here, if necessary.  This means
        # that you can have a single 'topic_images' folder serving multiple
        # weblog instances.
        topic_images = getattr(self, 'topic_images', None)
        if topic_images is None:
            return None
        normalizer = getUtility(IIDNormalizer)
        keyword_id = normalizer.normalize(self.keywords[0].decode('utf-8'))
        # To check for the presence of the image, we want to limit ourselves to
        # unacquired objects, so we use aq_base.
        image = getattr(aq_base(topic_images), keyword_id, None)
        if image is not None:
            # Now we've found it, we want to pass back an Implicit wrapper,
            # so we lookup the object again.
            image = getattr(topic_images, keyword_id)
        else:
            # Fallback for older Quills releases which did not normalize
            # the keyword to a safe Zope id.
            # Again, limit to unacquired objects
            image = getattr(aq_base(topic_images), self.keywords[0], None)
            if image is not None:
                # And then get the Implicit wrapper
                image = getattr(topic_images, self.keywords[0])
        return image

    def getEntries(self, maximum=None, offset=0):
        """See ITopic.
        """
        weblog = self.getWeblogContentObject()
        weblog_config = IWeblogEnhancedConfiguration(weblog)
        path = '/'.join(weblog.getPhysicalPath())
        catalog = getToolByName(self, 'portal_catalog')
        ifaces = [interfaceToName(catalog.aq_parent, IWeblogEntry),
                  interfaceToName(catalog.aq_parent, IPossibleWeblogEntry)]
        results = catalog(
                object_provides={'query' : ifaces, 'operator' : 'or'},
                path={'query':path, 'level': 0},
                Subject={'query'    : self.keywords,
                         'operator' : 'and'},
                sort_on='effective',
                sort_order='reverse',
                review_state=weblog_config.published_states)
        results = results[offset:]
        if maximum is not None:
            results = results[:maximum]
        return BloggifiedCatalogResults(results)

    def __str__(self):
        """See ITopic.
        """
        return self.keywords[0]

    def __len__(self):
        """See ITopic.
        """
        return len(self.getEntries())


class AuthorTopic(Topic):
    """ Filter post by author. Joining semantic for filtering by multiple
    authors is “or”.

    XXX Deriving from Topic is awkward, as an AuthorTopic is logically
    no keyword topic.
    """

    implements(IAuthorTopic)

    def getTitle(self):
        """See ITopic.
        """
        memb_tool = getToolByName(self, 'portal_membership')
        users = []
        for user_id in self.keywords:
            info = memb_tool.getMemberInfo(user_id)
            if info is None:
                users.append(user_id)
            else:
                fullname = info['fullname'] or user_id
                users.append(fullname)               
        return ", ".join(users).decode('utf-8')

    def getDescription(self):
        """See ITopic.
        """
        # XXX Fix me with something sensible.
        # Get the description from the member object?
        return ''

    def getImage(self):
        """See ITopic.
        """
        memb_tool = getToolByName(self, 'portal_membership')
        return memb_tool.getPersonalPortrait(self.keywords[0])

    def getEntries(self):
        """See ITopic.
        """
        weblog = self.getWeblogContentObject()
        weblog_config = IWeblogEnhancedConfiguration(weblog)
        path = '/'.join(weblog.getPhysicalPath())
        catalog = getToolByName(self, 'portal_catalog')
        ifaces = [interfaceToName(catalog.aq_parent, IWeblogEntry),
                  interfaceToName(catalog.aq_parent, IPossibleWeblogEntry)]
        results = catalog(
                object_provides={'query' : ifaces, 'operator' : 'or'},
                path={'query':path, 'level': 0},
                Creator={'query'    : self.keywords,
                         'operator' : 'or'},
                sort_on='effective',
                sort_order='reverse',
                review_state=weblog_config.published_states)
        return BloggifiedCatalogResults(results)


class TopicContainer(QuillsMixin, AcquiringActionProvider, Traversable,
                     Implicit):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(ITopicContainer, TopicContainer)
    True
    """

    implements(ITopicContainer, ITransientTopicContainer)

    __allow_access_to_unprotected_subobjects__ = EvilAATUSHack()

    def __init__(self, id, title='Topics'):
        self._id = str(id)
        self._title = title

    def getId(self):
        """
        """
        return self._id

    def Title(self):
        """
        """
        return self._title

    def getTopics(self):
        """See ITopicContainer.
        """
        keywords = self._getKeywordsForBlogEntries()
        return [Topic(kw).__of__(self) for kw in keywords]

    def getTopicById(self, id):
        """See ITopicContainer.
        """
        return Topic(id).__of__(self)

    def _getKeywordsForBlogEntries(self):
        """Return a sequence of all keywords that are associatd with
        IWeblogEntry instances contained in this IWeblog.
        """
        entries = self.getWeblog().getAllEntries()
        # Use dict rather than list to avoid duplicates
        keywords = {}
        for entry in entries:
            for kwds in entry.getTopics():
                for kw in kwds.getKeywords():
                    keywords[kw] = None
        keys = keywords.keys()
        keys.sort()
        return keys



class AuthorContainer(QuillsMixin, AcquiringActionProvider, Traversable,
                     Implicit):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IAuthorContainer, AuthorContainer)
    True
    """

    implements(IAuthorContainer, ITransientAuthorContainer)

    __allow_access_to_unprotected_subobjects__ = EvilAATUSHack()

    def __init__(self, id, title='Authors'):
        self._id = str(id)
        self._title = title

    def getId(self):
        """
        """
        return self._id

    def Title(self):
        """
        """
        return self._title

    def getAuthors(self):
        """See IAuthorContainer.
        """
        authors = self._getAuthorsForBlogEntries()
        return [AuthorTopic(author).__of__(self) for author in authors]

    def getAuthorById(self, id):
        """See IAuthorContainer.
        """
        return AuthorTopic(id).__of__(self)

    def _getAuthorsForBlogEntries(self):
        """Return a sequence of all keywords that are associatd with
        IWeblogEntry instances contained in this IWeblog.
        """
        entries = self.getWeblog().getEntries()
        # Use dict rather than list to avoid duplicates
        authors = {}
        for entry in entries:
            for author in entry.getAuthors():
                authors[author.getId()] = None
        authors = authors.keys()
        authors.sort()
        return authors



# Zope imports
from zope.formlib import form
from zope.interface import implements
from AccessControl import getSecurityManager
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

# CMF imports
from Products.CMFCore.permissions import AddPortalContent

# Plone imports
from plone.app.portlets.portlets import base
from plone.app.portlets.browser.formhelper import NullAddForm
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider

# Quills imports
from quills.core.interfaces import IWeblogLocator
from quills.app.interfaces import IWeblogEnhancedConfiguration
from quills.app import QuillsAppMessageFactory as _

# Local imports
from base import BasePortletRenderer


PORTLET_TITLE = _(u"Weblog Admin")
PORTLET_DESC = _(u"This portlet provides useful admin functions for a weblog.")


class IWeblogAdminPortlet(IPortletDataProvider):
    """A weblog administration portlet.
    """


class Assignment(base.Assignment):
    implements(IWeblogAdminPortlet)

    @property
    def title(self):
        return PORTLET_TITLE


class Renderer(BasePortletRenderer, base.Renderer):

    _template = ViewPageTemplateFile('weblogadmin.pt')

    @property
    def available(self):
        # This admin portlet is only available to people with the
        # 'Add portal content' permission.
        user = getSecurityManager().getUser()
        # I think has_permission sometimes returns 1 or None, so make sure that
        # we return True/False.
        if user.has_permission(AddPortalContent, self.context.aq_inner):
            # XXX Before returning True, we should also check the
            # BasePortletRenderer.available implementation. Not sure how to do
            # that as it's been property-ized on the base class.
            #return True
            return super(Renderer, self).available
        return False

    @property
    def title(self):
        return PORTLET_TITLE

    @property
    def add_entry_url(self):
        weblog_content = self.getWeblogContentObject()
        try:
            config = IWeblogEnhancedConfiguration(weblog_content)
            type_name = config.default_type
        except TypeError: # Could not adapt, so fall back to default.
            type_name = 'WeblogEntry'
        url = "%s/createObject?type_name=%s"
        return url % (weblog_content.absolute_url(), type_name)

    @property
    def drafts_url(self):
        weblog_content = self.getWeblogContentObject()
        return "%s/drafts/" % weblog_content.absolute_url()

    @property
    def manage_comments_url(self):
        weblog_content = self.getWeblogContentObject()
        return "%s/manage_comments" % weblog_content.absolute_url()

    @property
    def config_view_url(self):
        weblog_content = self.getWeblogContentObject()
        return "%s/config_view" % weblog_content.absolute_url()


class AddForm(NullAddForm):
    form_fields = form.Fields(IWeblogAdminPortlet)
    label = _(u'add-portlet', default=u"Add ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC

    def create(self):
        return Assignment()


class EditForm(base.EditForm):
    form_fields = form.Fields(IWeblogAdminPortlet)
    label = _(u'edit-portlet', default=u"Edit ${portlet-name} Portlet", mapping={u'portlet-name': PORTLET_TITLE})
    description = PORTLET_DESC
